@@ -10,10 +10,11 @@
 
 import sys
 
+import quodlibet
 from quodlibet.operon import main
-from quodlibet.util import set_win32_unicode_argv
+from quodlibet import util
 
 
 if __name__ == "__main__":
-    set_win32_unicode_argv()
-    sys.exit(main(sys.argv))
+    quodlibet.init_cli()
+    sys.exit(main(util.argv))
