@@ -18,11 +18,12 @@ from quodlibet import config
 from quodlibet import formats
 from quodlibet import qltk
 
-from quodlibet.browsers._base import Browser
+from quodlibet.browsers import Browser
 from quodlibet.library import SongFileLibrary
 from quodlibet.qltk.filesel import MainDirectoryTree
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.x import ScrolledWindow
+from quodlibet.qltk import Icons
 from quodlibet.util import copool
 from quodlibet.util.library import get_scan_dirs
 from quodlibet.util.dprint import print_d
@@ -32,12 +33,12 @@ from quodlibet.util import connect_obj
 
 
 class FileSystem(Browser, Gtk.HBox):
-    __gsignals__ = Browser.__gsignals__
 
     __library = None
 
     name = _("File System")
     accelerated_name = _("_File System")
+    keys = ["FileSystem"]
     priority = 10
     uses_main_library = False
 
@@ -119,7 +120,7 @@ class FileSystem(Browser, Gtk.HBox):
             qltk.selection_set_songs(sel, songs)
         else:
             # External target (app) is delivered a list of URIS of songs
-            uris = list(set([URI.frompath(dir) for dir in dirs]))
+            uris = list({URI.frompath(dir) for dir in dirs})
             print_d("Directories to drop: %s" % [u.filename for u in uris])
             sel.set_uris(uris)
 
@@ -183,7 +184,7 @@ class FileSystem(Browser, Gtk.HBox):
 
     def Menu(self, songs, library, items):
 
-        i = qltk.MenuItem(_("_Add to Library"), Gtk.STOCK_ADD)
+        i = qltk.MenuItem(_("_Add to Library"), Icons.LIST_ADD)
         i.set_sensitive(False)
         i.connect('activate', self.__add_songs, songs)
         for song in songs:
