@@ -14,30 +14,30 @@ from quodlibet import config
 from quodlibet import qltk
 from quodlibet import util
 
-from quodlibet.browsers._base import Browser
+from quodlibet.browsers import Browser
 from quodlibet.formats import PEOPLE
-from quodlibet.parse import Query
+from quodlibet.query import Query
 from quodlibet.qltk.songlist import SongList
 from quodlibet.qltk.completion import LibraryTagCompletion
 from quodlibet.qltk.searchbar import SearchBarBox
-from quodlibet.qltk.x import ScrolledWindow, Alignment, RPaned
+from quodlibet.qltk.x import ScrolledWindow, Align
 from quodlibet.util.library import background_filter
+from quodlibet.util import connect_destroy
 
 from .prefs import PreferencesButton
 from .util import get_headers
 from .pane import Pane
 
 
-class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
+class PanedBrowser(Browser, util.InstanceTracker):
     """A Browser enabling "drilling down" of tracks by successive
     selections in multiple tag pattern panes (e.g. Genre / People / Album ).
     It presents available values (and track counts) for each pane's tag
     """
 
-    __gsignals__ = Browser.__gsignals__
-
     name = _("Paned Browser")
     accelerated_name = _("_Paned Browser")
+    keys = ["Paned", "PanedBrowser"]
     priority = 3
 
     def pack(self, songpane):
@@ -62,17 +62,15 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
             browser.refresh_panes()
             browser.fill_panes()
 
-    def __init__(self, library, main):
+    def __init__(self, library):
         super(PanedBrowser, self).__init__()
         self._register_instance()
 
-        self.__main = main
-
-        self._filter = None
+        self._filter = lambda s: False
         self._library = library
-        self.commands = {"query": self.__query}
 
         self.set_spacing(6)
+        self.set_orientation(Gtk.Orientation.VERTICAL)
 
         completion = LibraryTagCompletion(library.librarian)
         self.accelerators = Gtk.AccelGroup()
@@ -82,32 +80,26 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         sbb.connect('focus-out', self.__focus)
         self._sb_box = sbb
 
-        align = (Alignment(sbb, left=6, right=6, top=6) if main
-                 else Alignment(sbb))
+        align = Align(sbb, left=6, right=6, top=6)
         self.pack_start(align, False, True, 0)
 
-        keyval, mod = Gtk.accelerator_parse("<control>Home")
-        s = self.accelerators.connect(keyval, mod, 0, self.__select_all)
-        self.connect_object('destroy',
-                            self.accelerators.disconnect_key, keyval, mod)
+        keyval, mod = Gtk.accelerator_parse("<Primary>Home")
+        self.accelerators.connect(keyval, mod, 0, self.__select_all)
         select = Gtk.Button(label=_("Select _All"), use_underline=True)
-        s = select.connect('clicked', self.__select_all)
-        self.connect_object('destroy', select.disconnect, s)
+        select.connect('clicked', self.__select_all)
         sbb.pack_start(select, False, True, 0)
 
         prefs = PreferencesButton(self)
         sbb.pack_start(prefs, False, True, 0)
 
-        for s in [library.connect('changed', self.__changed),
-                  library.connect('added', self.__added),
-                  library.connect('removed', self.__removed)
-                  ]:
-            self.connect_object('destroy', library.disconnect, s)
+        connect_destroy(library, 'changed', self.__changed)
+        connect_destroy(library, 'added', self.__added)
+        connect_destroy(library, 'removed', self.__removed)
 
         self.connect('destroy', self.__destroy)
 
         # contains the panes and the song list
-        self.main_box = RPaned()
+        self.main_box = qltk.ConfigRPaned("browsers", "panedbrowser_pos", 0.4)
         self.pack_start(self.main_box, True, True, 0)
 
         self.refresh_panes()
@@ -116,7 +108,6 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
             child.show_all()
 
     def __destroy(self, *args):
-        del self.commands
         del self._sb_box
 
     def set_wide_mode(self, do_wide):
@@ -137,9 +128,6 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
     def _set_text(self, text):
         self._sb_box.set_text(text)
 
-    def __query(self, text, library, window, player):
-        self.filter_text(text)
-
     def __focus(self, widget, *args):
         qltk.get_top_parent(widget).songlist.grab_focus()
 
@@ -150,6 +138,9 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         self._set_text(text)
         self.activate()
 
+    def get_filter_text(self):
+        return self._get_text()
+
     def __select_all(self, *args):
         self._panes[-1].inhibit()
         for pane in self._panes:
@@ -217,9 +208,8 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         self._panes.pop()  # remove self
 
         for pane in self._panes:
-            if self.__main:
-                pane.connect('row-activated',
-                             lambda *x: self.emit("activated"))
+            pane.connect('row-activated',
+                         lambda *x: self.songs_activated())
             sw = ScrolledWindow()
             sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
             sw.set_shadow_type(Gtk.ShadowType.IN)
@@ -323,4 +313,4 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
             self.fill_panes()
 
     def fill(self, songs):
-        GLib.idle_add(self.emit, 'songs-selected', list(songs), None)
+        GLib.idle_add(self.songs_selected, list(songs))
