@@ -12,14 +12,15 @@ from gi.repository import Gtk, GLib
 from quodlibet import config
 from quodlibet import qltk
 
-from quodlibet.browsers._base import Browser
-from quodlibet.parse import Query
+from quodlibet.browsers import Browser
+from quodlibet.query import Query
 from quodlibet.qltk.ccb import ConfigCheckMenuItem
 from quodlibet.qltk.completion import LibraryTagCompletion
 from quodlibet.qltk.menubutton import MenuButton
 from quodlibet.qltk.songlist import SongList
 from quodlibet.qltk.searchbar import LimitSearchBarBox
-from quodlibet.qltk.x import Alignment, SymbolicIconImage
+from quodlibet.qltk.x import Align, SymbolicIconImage
+from quodlibet.qltk import Icons
 
 
 class PreferencesButton(Gtk.HBox):
@@ -35,21 +36,19 @@ class PreferencesButton(Gtk.HBox):
         menu.show_all()
 
         button = MenuButton(
-            SymbolicIconImage("emblem-system", Gtk.IconSize.MENU),
+            SymbolicIconImage(Icons.EMBLEM_SYSTEM, Gtk.IconSize.MENU),
             arrow=True)
         button.set_menu(menu)
         self.pack_start(button, True, True, 0)
 
 
-class SearchBar(Gtk.VBox, Browser):
+class SearchBar(Browser):
     """Like EmptyBar, but the user can also enter a query manually"""
 
-    __gsignals__ = Browser.__gsignals__
-
     name = _("Search Library")
     accelerated_name = _("_Search Library")
+    keys = ["SearchBar"]
     priority = 1
-    in_menu = True
 
     def pack(self, songpane):
         container = Gtk.VBox(spacing=6)
@@ -61,13 +60,13 @@ class SearchBar(Gtk.VBox, Browser):
         container.remove(songpane)
         container.remove(self)
 
-    def __init__(self, library, main):
+    def __init__(self, library):
         super(SearchBar, self).__init__()
         self.set_spacing(6)
+        self.set_orientation(Gtk.Orientation.VERTICAL)
 
-        self._filter = None
+        self._query = None
         self._library = library
-        self.commands = {"query": self.__query}
 
         completion = LibraryTagCompletion(library.librarian)
         self.accelerators = Gtk.AccelGroup()
@@ -84,8 +83,7 @@ class SearchBar(Gtk.VBox, Browser):
         prefs = PreferencesButton(sbb)
         sbb.pack_start(prefs, False, True, 0)
 
-        align = (Alignment(sbb, left=6, right=6, top=6) if main
-                 else Alignment(sbb))
+        align = Align(sbb, left=6, right=6, top=6)
         self.pack_start(align, False, True, 0)
         self.connect('destroy', self.__destroy)
         self.show_all()
@@ -98,10 +96,6 @@ class SearchBar(Gtk.VBox, Browser):
 
     def __destroy(self, *args):
         self._sb_box = None
-        del self.commands
-
-    def __query(self, text, library, window, player):
-        self.filter_text(text)
 
     def __focus(self, widget, *args):
         qltk.get_top_parent(widget).songlist.grab_focus()
@@ -109,22 +103,17 @@ class SearchBar(Gtk.VBox, Browser):
     def _get_songs(self):
         text = self._get_text()
         try:
-            self._filter = Query(text, star=SongList.star).search
+            self._query = Query(text, star=SongList.star)
         except Query.error:
             pass
         else:
-            if Query.match_all(text):
-                songs = self._library.values()
-                self._filter = None
-            else:
-                songs = filter(self._filter, self._library)
-            return songs
+            return self._query.filter(self._library)
 
     def activate(self):
         songs = self._get_songs()
         if songs is not None:
             songs = self._sb_box.limit(songs)
-            GLib.idle_add(self.emit, 'songs-selected', songs, None)
+            GLib.idle_add(self.songs_selected, songs)
 
     def __text_parse(self, bar, text):
         self.activate()
@@ -150,12 +139,15 @@ class SearchBar(Gtk.VBox, Browser):
         self._set_text(text)
         self.activate()
 
+    def get_filter_text(self):
+        return self._get_text()
+
     def unfilter(self):
         self.filter_text("")
 
     def active_filter(self, song):
-        if self._filter is not None:
-            return self._filter(song)
+        if self._query is not None:
+            return self._query.search(song)
         else:
             return True
 
