@@ -9,7 +9,7 @@
 
 import shutil
 
-import const
+from . import const
 from quodlibet.util.config import Config, Error
 
 # Some plugins can be enabled on first install
@@ -67,6 +67,7 @@ INITIAL = {
         "collection_headers": "~people 0",
         "radio": "", # radio filter selection
         "rating_click": "true", # click to rate song, on/off
+        "rating_hotkeys": "true", # press number keys to rate song on/off
         "rating_confirm_multiple": "false", # confirm rating multiple songs
         "cover_size": "-1", # max cover height/width, <= 0 is default
         "search_limit": "false", # Show the limit widgets for SearchBar
@@ -114,6 +115,10 @@ INITIAL = {
         # tags which get searched in addition to the ones present in the
         # song list, separate with ","
         "search_tags": "",
+
+        # If set to "true" allow directly deleting files, even on systems that
+        # support sending them to the trash.
+        "bypass_trash": "false",
     },
     "rename": {
         "spaces": "false",
@@ -165,7 +170,6 @@ setstringlist = _config.setstringlist
 getlist = _config.getlist
 setlist = _config.setlist
 set = _config.set
-setdefault = _config.setdefault
 write = _config.write
 reset = _config.reset
 add_section = _config.add_section
@@ -173,20 +177,27 @@ has_option = _config.has_option
 remove_option = _config.remove_option
 register_upgrade_function = _config.register_upgrade_function
 
+_filename = None
+"""The filename last used for loading"""
 
-def init(filename=None, initial=None):
-    if not _config.is_empty():
-        raise ValueError(
-            "config initialized twice without quitting: %r"
-            % _config.sections())
 
-    if initial is None:
-        initial = INITIAL
+def init_defaults():
+    """Fills in the defaults, so they are guaranteed to be available"""
 
-    for section, values in initial.iteritems():
-        _config.add_section(section)
+    _config.defaults.clear()
+    for section, values in INITIAL.iteritems():
+        _config.defaults.add_section(section)
         for key, value in values.iteritems():
-            _config.set_inital(section, key, value)
+            _config.defaults.set(section, key, value)
+
+
+def init(filename=None):
+    global _filename
+
+    if not _config.is_empty():
+        raise ValueError("config initialized twice without quitting")
+
+    _filename = filename
 
     if filename is not None:
         try:
@@ -201,8 +212,18 @@ def init(filename=None, initial=None):
                 pass
 
 
-def save(filename):
-    """Writes the active config to filename, ignoring all possible errors"""
+def save(filename=None):
+    """Writes the active config to filename, ignoring all possible errors.
+
+    If not filename is given the one used for loading is used.
+    """
+
+    global _filename
+
+    if filename is None:
+        filename = _filename
+        if filename is None:
+            return
 
     print_d("Writing config...")
     try:
