@@ -14,25 +14,27 @@ from quodlibet.util.dprint import print_d, print_w
 from gi.repository import GObject
 from quodlibet.util.path import xdg_get_system_data_dirs
 
-try:
-    import dbus
-except ImportError:
-    print_w(_("Could not import %s, which is needed for "
-        "device support.") % "dbus-python")
-    dbus = None
-
-from quodlibet import const
+import quodlibet
 from quodlibet import util
 from quodlibet.devices import _udev as udev
 from quodlibet.util.importhelper import load_dir_modules
 
+try:
+    import dbus
+except ImportError:
+    if not util.is_osx() and not util.is_windows():
+        print_w(_("Could not import %s, which is needed for "
+            "device support.") % "dbus-python")
+    dbus = None
+
+
 devices = []
 
 
 def init_devices():
     global devices
 
-    load_pyc = os.name == 'nt'
+    load_pyc = util.is_windows() or util.is_osx()
     modules = load_dir_modules(dirname(__file__),
                                package=__package__,
                                load_compiled=load_pyc)
@@ -45,9 +47,11 @@ def init_devices():
 
     devices.sort()
 
-init_devices()
 
-DEVICES = os.path.join(const.USERDIR, "devices")
+if not util.is_osx() and not util.is_windows():
+    init_devices()
+
+DEVICES = os.path.join(quodlibet.get_user_dir(), "devices")
 
 config = ConfigParser.RawConfigParser()
 config.read(DEVICES)
@@ -126,6 +130,9 @@ class DeviceManager(GObject.GObject):
         the device serial (so it's unique) and maybe the model name."""
         device = None
 
+        print_d("Creating device %r supporting protocols: %r" % (
+            device_id, protocols))
+
         for prots in (protocols, ['storage']):
             klass = get_by_protocols(prots)
             if klass is None:
@@ -143,8 +150,12 @@ class DeviceManager(GObject.GObject):
         return device
 
 
-def get_device_from_path(udev_ctx, path):
-    """A dict of device attributes for the given device path"""
+def get_devices_from_path(udev_ctx, path):
+    """A list of device attribute dicts for the given device path and all its
+    parents.
+
+    Either returns a non empty list or raises EnvironmentError.
+    """
 
     path = path.encode("ascii")
     enum = udev.UdevEnumerate.new(udev_ctx)
@@ -153,12 +164,12 @@ def get_device_from_path(udev_ctx, path):
         raise EnvironmentError
 
     # only match the device we want
-    if enum.add_match_property("DEVNAME", path):
+    if enum.add_match_property("DEVNAME", path) != 0:
         enum.unref()
         raise EnvironmentError
 
     # search for it
-    if enum.scan_devices():
+    if enum.scan_devices() != 0:
         enum.unref()
         raise EnvironmentError
 
@@ -167,59 +178,285 @@ def get_device_from_path(udev_ctx, path):
     if not entry:
         enum.unref()
         raise EnvironmentError
-
-    device = udev.UdevDevice.new_from_syspath(udev_ctx, entry.get_name())
-    if not device:
-        enum.unref()
-        raise EnvironmentError
-
-    entry = device.get_properties_list_entry()
-    if not entry:
-        enum.unref()
-        device.unref()
-        raise EnvironmentError
-
-    attrs = {}
-    for e in entry:
-        name = e.get_name()
-        value = e.get_value()
-        attrs[name] = value.decode("string-escape")
-
+    sys_path = entry.get_name()
     enum.unref()
-    device.unref()
-    return attrs
+
+    device = udev.UdevDevice.new_from_syspath(udev_ctx, sys_path)
+    if not device:
+        raise EnvironmentError
+
+    devices = []
+    while device:
+        devices.append(device)
+        device = device.get_parent()
+
+    device_attrs = []
+    for device in devices:
+        entry = device.get_properties_list_entry()
+        if not entry:
+            continue
+
+        attrs = {}
+        for e in entry:
+            name = e.get_name()
+            value = e.get_value()
+            attrs[name] = value.decode("string-escape")
+        device_attrs.append(attrs)
+
+    # the first device owns its parents
+    devices[0].unref()
+
+    return device_attrs
 
 
-class DKD(DeviceManager):
-    __interface = None
-    __udev = None
+def dbus_barray_to_str(array):
+    return b"".join(map(bytes, array)).rstrip(b"\x00")
 
-    def __init__(self, dkd_name):
-        self.__bus = ".".join(dkd_name)
-        self.__path = "/".join(dkd_name)
-        super(DKD, self).__init__("org.freedesktop.%s" % self.__bus)
+
+class UDisks2Manager(DeviceManager):
+
+    BUS_NAME = "org.freedesktop.UDisks2"
+    BLOCK_IFACE = "org.freedesktop.UDisks2.Block"
+    FS_IFACE = "org.freedesktop.UDisks2.Filesystem"
+    DRIVE_IFACE = "org.freedesktop.UDisks2.Drive"
+    PROP_IFACE = "org.freedesktop.DBus.Properties"
+    OBJMAN_IFACE = "org.freedesktop.DBus.ObjectManager"
+
+    def __init__(self):
+        super(UDisks2Manager, self).__init__(self.BUS_NAME)
 
         error = False
 
         try:
             udev.init()
         except OSError:
-            print_w(_("%s: Could not find 'libudev'.") % self.__bus)
+            print_w("UDisks2: " + _("Could not find '%s'.") % "libudev")
             error = True
         else:
-            self.__udev = udev.Udev.new()
+            self._udev = udev.Udev.new()
 
-        if self.__get_mpi_dir() is None:
-            print_w(_("%s: Could not find %s.")
-                    % (self.__bus, "media-player-info"))
+        if get_mpi_dir() is None:
+            print_w("UDisks2: " + _("Could not find '%s'.")
+                    % "media-player-info")
             error = True
 
         if error:
             raise LookupError
 
-        interface = "org.freedesktop.%s" % self.__bus
-        path = "/org/freedesktop/%s" % self.__path
-        obj = self._system_bus.get_object(interface, path)
+        # object path -> properties
+        self._drives = {}
+        self._fs = {}
+        self._blocks = {}
+
+        # object paths -> devices (block/fs) we emitted
+        self._devices = {}
+
+        obj = self._system_bus.get_object(
+            self.BUS_NAME, "/org/freedesktop/UDisks2")
+        self._interface = dbus.Interface(obj, self.OBJMAN_IFACE)
+        self._interface.connect_to_signal(
+            'InterfacesAdded', self._interface_added)
+        self._interface.connect_to_signal(
+            'InterfacesRemoved', self._interface_removed)
+
+    def _try_build_device(self, object_path, block, fs):
+        """Returns a Device instance or None.
+
+        None if it wasn't a media player etc..
+        """
+
+        drive = self._drives.get(block["Drive"])
+        if not drive:
+            # I think this shouldn't happen, but check anyway
+            return
+
+        dev_path = dbus_barray_to_str(block["Device"])
+        print_d("Found device: %r" % dev_path)
+
+        media_player_id = get_media_player_id(self._udev, dev_path)
+        if not media_player_id:
+            print_d("%r not a media player" % dev_path)
+            return
+        protocols = get_media_player_protocols(media_player_id)
+
+        device_id = drive["Id"]
+
+        dev = self.create_device(object_path, unicode(device_id), protocols)
+        icon_name = block["HintIconName"]
+        if icon_name:
+            dev.icon = icon_name
+        return dev
+
+    def discover(self):
+        objects = self._interface.GetManagedObjects()
+        for object_path, interfaces_and_properties in objects.iteritems():
+            self._update_interfaces(object_path, interfaces_and_properties)
+        self._check_interfaces()
+
+    def get_name(self, path):
+        block = self._blocks[path]
+        drive = self._drives.get(block["Drive"])
+        return " - ".join([drive["Vendor"], drive["Model"]])
+
+    def get_mountpoint(self, path):
+        # the mointpoint gets filed with a delay, so fetch over dbus each time
+        # the get the correct value
+        obj = self._system_bus.get_object(self.BUS_NAME, path)
+        interface = dbus.Interface(obj, self.PROP_IFACE)
+        try:
+            array = interface.Get(self.FS_IFACE, "MountPoints")
+        except dbus.DBusException:
+            paths = []
+        else:
+            paths = [dbus_barray_to_str(v) for v in array]
+
+        if paths:
+            return paths[0]
+        return ""
+
+    def get_block_device(self, path):
+        block = self._blocks[path]
+        return dbus_barray_to_str(block["Device"])
+
+    def eject(self, path):
+        # first try to unmount
+        obj = self._system_bus.get_object(self.BUS_NAME, path)
+        interface = dbus.Interface(obj, self.FS_IFACE)
+        try:
+            interface.Unmount({})
+        except dbus.DBusException:
+            pass
+
+        # then eject..
+        # XXX: this only works if no other FS is mounted..
+        block = self._blocks[path]
+        obj = self._system_bus.get_object(self.BUS_NAME, block["Drive"])
+        interface = dbus.Interface(obj, self.DRIVE_IFACE)
+        try:
+            interface.Eject({})
+        except dbus.DBusException:
+            return False
+        return True
+
+    def _update_interfaces(self, object_path, iap):
+        if self.DRIVE_IFACE in iap:
+            self._drives[object_path] = iap[self.DRIVE_IFACE]
+        if self.FS_IFACE in iap:
+            self._fs[object_path] = iap[self.FS_IFACE]
+        if self.BLOCK_IFACE in iap:
+            self._blocks[object_path] = iap[self.BLOCK_IFACE]
+
+    def _check_interfaces(self):
+        # we need the block and fs interface to create a device
+        for object_path in (set(self._fs.keys()) & set(self._blocks.keys())):
+            # we are finished with this one, ignore
+            if object_path in self._devices:
+                continue
+            block = self._blocks[object_path]
+            fs = self._fs[object_path]
+            dev = self._try_build_device(object_path, block, fs)
+            if dev:
+                self._devices[object_path] = dev
+                self.emit("added", dev)
+
+    def _interface_added(self, object_path, iap):
+        self._update_interfaces(object_path, iap)
+        self._check_interfaces()
+
+    def _interface_removed(self, object_path, interfaces):
+        if self.FS_IFACE in interfaces or self.BLOCK_IFACE in interfaces:
+            # if any of our needed interfaces goes away, remove the device
+            if object_path in self._devices:
+                self.emit("removed", unicode(object_path))
+                dev = self._devices[object_path]
+                dev.close()
+                del self._devices[object_path]
+
+        if self.DRIVE_IFACE in interfaces:
+            del self._drives[object_path]
+        if self.FS_IFACE in interfaces:
+            del self._fs[object_path]
+        if self.BLOCK_IFACE in interfaces:
+            del self._blocks[object_path]
+
+
+def get_media_player_id(udev_ctx, dev_path):
+    """Get the ID_MEDIA_PLAYER key for a specific device path e.g. /dev/sdc
+
+    Returns the str ID or None.
+    """
+
+    try:
+        devs = get_devices_from_path(udev_ctx, dev_path)
+    except Exception:
+        print_w("Failed to retrieve udev properties for %r" % dev_path)
+        util.print_exc()
+        return
+
+    for dev in devs:
+        try:
+            return dev["ID_MEDIA_PLAYER"]
+        except KeyError:
+            continue
+
+
+def get_mpi_dir():
+    """Path to the media-player-info directory or None"""
+
+    for dir_ in xdg_get_system_data_dirs():
+        mpi_path = os.path.join(dir_, "media-player-info")
+        if os.path.isdir(mpi_path):
+            return mpi_path
+
+
+def get_media_player_protocols(media_player_id):
+    """Gives a list of supported protocols"""
+
+    # get the path to the mpi files
+    mpi_path = get_mpi_dir()
+    if not mpi_path:
+        return []
+
+    file_path = os.path.join(mpi_path, media_player_id + ".mpi")
+    parser = ConfigParser.SafeConfigParser()
+    if parser.read(file_path):
+        try:
+            prots = parser.get("Device", "AccessProtocol")
+        except ConfigParser.Error:
+            return []
+        else:
+            return prots.split(";")
+    return []
+
+
+class UDisks1Manager(DeviceManager):
+
+    def __init__(self):
+        bus_name = "org.freedesktop.UDisks"
+        interface = "org.freedesktop.UDisks"
+        path = "/org/freedesktop/UDisks"
+
+        super(UDisks1Manager, self).__init__(bus_name)
+
+        error = False
+
+        try:
+            udev.init()
+        except OSError:
+            print_w("UDisks: " + _("Could not find '%s'.") % "libudev")
+            error = True
+        else:
+            self.__udev = udev.Udev.new()
+
+        if get_mpi_dir() is None:
+            print_w("UDisks: " + _("Could not find '%s'.")
+                    % "media-player-info")
+            error = True
+
+        if error:
+            raise LookupError
+
+        obj = self._system_bus.get_object(bus_name, path)
         self.__interface = dbus.Interface(obj, interface)
 
         self.__devices = {}
@@ -228,20 +465,21 @@ class DKD(DeviceManager):
             self.__device_removed)
 
     def __get_dev_prop_interface(self, path):
-        interface = "org.freedesktop.%s" % self.__bus
-        obj = self._system_bus.get_object(interface, path)
+        bus_name = "org.freedesktop.UDisks"
+        obj = self._system_bus.get_object(bus_name, path)
         return dbus.Interface(obj, "org.freedesktop.DBus.Properties")
 
     def __get_dev_interface(self, path):
-        interface = "org.freedesktop.%s" % self.__bus
-        obj = self._system_bus.get_object(interface, path)
-        return dbus.Interface(obj, "org.freedesktop.%s.Device" % self.__bus)
+        bus_name = "org.freedesktop.UDisks"
+        obj = self._system_bus.get_object(bus_name, path)
+        return dbus.Interface(obj, "org.freedesktop.UDisks.Device")
 
     def __get_dev_property(self, interface, property):
         return interface.Get("org.freedesktop.DBus.Properties", property)
 
     def __get_device_id(self, path):
         """A unique device id"""
+
         prop_if = self.__get_dev_prop_interface(path)
         dev_id = self.__get_dev_property(prop_if, 'device-file-by-id')[0]
         dev_id = basename(dev_id)
@@ -261,6 +499,7 @@ class DKD(DeviceManager):
         self.emit("removed", path)
         dev = self.__devices[path]
         dev.close()
+        del self.__devices[path]
 
     def discover(self):
         paths = self.__interface.EnumerateDevices()
@@ -312,39 +551,6 @@ class DKD(DeviceManager):
         prop_if = self.__get_dev_prop_interface(path)
         return str(self.__get_dev_property(prop_if, 'device-file'))
 
-    def __get_media_player_id(self, devpath):
-        """DKD is for high-level device stuff. The info if the device is
-        a media player and what protocol/formats it supports can only
-        be retrieved through libudev"""
-        try:
-            dev = get_device_from_path(self.__udev, devpath)
-        except Exception:
-            print_w("Failed to retrieve udev properties for %r" % devpath)
-            util.print_exc()
-            return
-
-        try:
-            return dev["ID_MEDIA_PLAYER"]
-        except KeyError:
-            return None
-
-    def __get_mpi_dir(self):
-        for dir in xdg_get_system_data_dirs():
-            path = os.path.join(dir, "media-player-info")
-            if os.path.isdir(path):
-                return path
-
-    def __get_mpi_file(self, dir, mplayer_id):
-        """Returns a SafeConfigParser instance of the mpi file or None.
-        MPI files are INI like files usually located in
-        /usr/local/media-player-info/*.mpi"""
-        f = os.path.join(dir, mplayer_id + ".mpi")
-        if os.path.isfile(f):
-            parser = ConfigParser.SafeConfigParser()
-            read = parser.read(f)
-            if read:
-                return parser
-
     def __build_dev(self, path):
         """Return the right device instance by determining the
         supported AccessProtocol"""
@@ -366,24 +572,15 @@ class DKD(DeviceManager):
                 int(prop_get(prop_if, 'partition-type'), 16) == 0:
             return
 
-        #ask libudev if the device is a media player
+        # ask libudev if the device is a media player
+        # and get supported protocols if any
         devpath = self.get_block_device(path)
-        mplayer_id = self.__get_media_player_id(devpath)
-        if mplayer_id is None:
+        media_player_id = get_media_player_id(self.__udev, devpath)
+        if not media_player_id:
             return
+        protocols = get_media_player_protocols(media_player_id)
 
-        #look up the supported protocols in the mpi files
-        protocols = []
-        mpi_dir = self.__get_mpi_dir()
-        config = self.__get_mpi_file(mpi_dir, mplayer_id)
-        if config is not None:
-            try:
-                prots = config.get("Device", "AccessProtocol")
-            except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
-                pass
-            else:
-                protocols = prots.split(";")
-
+        # unique id
         device_id = self.__get_device_id(path)
 
         dev = self.create_device(path, device_id, protocols)
@@ -402,19 +599,17 @@ def init():
     print_d(_("Initializing device backend."))
     try_text = _("Trying '%s'")
 
-    #DKD maintainers will change the naming of dbus, app stuff
-    #in january 2010 or so (already changed in trunk), so try both
     if device_manager is None:
-        print_d(try_text % "DeviceKit Disks")
+        print_d(try_text % "UDisks2")
         try:
-            device_manager = DKD(("DeviceKit", "Disks"))
+            device_manager = UDisks2Manager()
         except (LookupError, dbus.DBusException):
             pass
 
     if device_manager is None:
-        print_d(try_text % "UDisks")
+        print_d(try_text % "UDisks1")
         try:
-            device_manager = DKD(("UDisks",))
+            device_manager = UDisks1Manager()
         except (LookupError, dbus.DBusException):
             pass
 
