@@ -12,7 +12,8 @@ from glob import glob
 
 from gi.repository import Gtk, GLib, GdkPixbuf
 
-from quodlibet import const
+import quodlibet
+from quodlibet import util
 from quodlibet import app
 
 from quodlibet.devices._base import Device
@@ -22,7 +23,7 @@ from quodlibet.qltk.msg import ConfirmFileReplace
 from quodlibet.util.path import (mtime, escape_filename,
     strip_win32_incompat_from_path)
 
-CACHE = os.path.join(const.USERDIR, 'cache')
+CACHE = os.path.join(quodlibet.get_user_dir(), 'cache')
 
 
 class StorageDevice(Device):
@@ -124,8 +125,9 @@ class StorageDevice(Device):
             song.sanitize(target)
             self.__library.add([song])
             return song
-        except (OSError, IOError, GLib.GError), exc:
-            return str(exc).decode(const.ENCODING, 'replace')
+        except (OSError, IOError, GLib.GError) as exc:
+            encoding = util.get_locale_encoding()
+            return str(exc).decode(encoding, 'replace')
 
     def delete(self, parent_widget, song):
         try:
@@ -146,8 +148,9 @@ class StorageDevice(Device):
                     pass
 
             return True
-        except (OSError, IOError), exc:
-            return str(exc).decode(const.ENCODING, 'replace')
+        except (OSError, IOError) as exc:
+            encoding = util.get_locale_encoding()
+            return str(exc).decode(encoding, 'replace')
 
     def cleanup(self, wlb, action):
         self.__save_library()
