@@ -5,11 +5,10 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from gi.repository import Gtk
-
 from quodlibet import app
 from quodlibet import browsers
 from quodlibet.qltk.browser import LibraryBrowser
+from quodlibet.qltk import Icons
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 
 
@@ -17,7 +16,7 @@ class FilterBrowser(SongsMenuPlugin):
     PLUGIN_ID = 'filterbrowser'
     PLUGIN_NAME = _('Filter on Directory')
     PLUGIN_DESC = _("Filters on directory in a new browser window.")
-    PLUGIN_ICON = Gtk.STOCK_INDEX
+    PLUGIN_ICON = Icons.EDIT_SELECT_ALL
 
     def plugin_songs(self, songs):
         tag = "~dirname"
