@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
-#           2012-2013 Nick Boultbee
+#           2012-2014 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -13,58 +13,60 @@
 import os
 import shutil
 import time
-import re
-import collections
+
+from quodlibet import util
+from quodlibet import config
+from quodlibet.util.path import mkdir, fsdecode, mtime, expanduser, is_fsnative
+from quodlibet.util.path import normalize_path, fsnative, escape_filename
+from quodlibet.util.string import encode, decode, isascii
+
+from quodlibet.util.uri import URI
+from quodlibet.util import human_sort_key as human, capitalize
+
+from quodlibet.util.tags import TAG_ROLES, TAG_TO_SORT
 
 from ._image import ImageContainer
 
-from quodlibet import const
-from quodlibet import util
-from quodlibet import config
-from quodlibet.util.path import mkdir, fsdecode, mtime, expanduser
-from quodlibet.util.path import normalize_path, fsnative, escape_filename
-from quodlibet.util.string import encode
 
-from quodlibet.util.uri import URI
-from quodlibet.util import human_sort_key as human
-from quodlibet.util.dprint import print_d, print_w
+MIGRATE = {"~#playcount", "~#laststarted", "~#lastplayed", "~#added",
+           "~#skipcount", "~#rating", "~bookmark"}
+"""These get migrated if a song gets reloaded"""
 
-from quodlibet.util.cover.manager import cover_plugins
+PEOPLE = ["artist", "albumartist", "author", "composer", "~performers",
+          "originalartist", "lyricist", "arranger", "conductor"]
+"""Sources of the ~people tag, most important first"""
 
-# Used by __init__.py
-from quodlibet.util.tags import STANDARD_TAGS as USEFUL_TAGS
-from quodlibet.util.tags import MACHINE_TAGS
+INTERN_NUM_DEFAULT = {"~#lastplayed", "~#laststarted", "~#playcount",
+                      "~#skipcount", "~#length", "~#bitrate", "~#filesize"}
+"""Default to 0"""
 
-
-MIGRATE = frozenset(("~#playcount ~#laststarted ~#lastplayed ~#added "
-           "~#skipcount ~#rating ~bookmark").split())
-
-PEOPLE = ("albumartist artist author composer ~performers originalartist "
-          "lyricist arranger conductor").split()
-
-TAG_ROLES = {
-    "composer": _("Composition"),
-    "lyricist": _("Lyrics"),
-    "arranger": _("Arrangement"),
-    "conductor": _("Conducting")
-}
-
-TAG_TO_SORT = {
-    "artist": "artistsort",
-    "album": "albumsort",
-    "albumartist": "albumartistsort",
-    "performer": "performersort",
-    "~performers": "~performerssort"
-}
-
-INTERN_NUM_DEFAULT = frozenset("~#lastplayed ~#laststarted ~#playcount "
-    "~#skipcount ~#length ~#bitrate ~#filesize".split())
+FILESYSTEM_TAGS = {"~filename", "~basename", "~dirname"}
+"""Values are bytes in Linux instead of unicode"""
 
 SORT_TO_TAG = dict([(v, k) for (k, v) in TAG_TO_SORT.iteritems()])
+"""Reverse map, so sort tags can fall back to the normal ones"""
 
 PEOPLE_SORT = [TAG_TO_SORT.get(k, k) for k in PEOPLE]
+"""Sources for ~peoplesort, most important first"""
 
-FILESYSTEM_TAGS = "~filename ~basename ~dirname".split()
+VARIOUS_ARTISTS_VALUES = 'V.A.', 'various artists', 'Various Artists'
+"""Values for ~people representing lots of people, most important last"""
+
+
+def decode_value(tag, value):
+    """Returns a unicode representation of the passed value, based on
+    the type and the tag it originated from.
+
+    Not reversible.
+    """
+
+    if isinstance(value, unicode):
+        return value
+    elif isinstance(value, float):
+        return u"%.2f" % value
+    elif tag in FILESYSTEM_TAGS:
+        return fsdecode(value)
+    return unicode(value)
 
 
 class AudioFile(dict, ImageContainer):
@@ -73,10 +75,20 @@ class AudioFile(dict, ImageContainer):
     __getitem__, get, and so on can be used for efficiency.
 
     If you need to sort many AudioFiles, you can use their sort_key
-    attribute as a decoration."""
+    attribute as a decoration.
+
+    Keys are either ASCII str or unicode.
+    Values are always unicode except if the tag is part of FILESYSTEM_TAGS,
+    then the value is of the path type (str on UNIX, unicode on Windows)
+
+    Some methods will make sure the returned values are always unicode, see
+    their description.
+    """
 
     # New tags received from the backend will update the song
     fill_metadata = False
+    # New song duration from the backend will update the song
+    fill_length = False
     # Container for multiple songs, while played new songs can start/end
     multisong = False
     # Part of a multisong
@@ -85,13 +97,10 @@ class AudioFile(dict, ImageContainer):
     can_add = True
     # Is a real file
     is_file = True
-    # Multiple tags for the same tag possible
-    multiple_values = True
 
     format = "Unknown Audio File"
     mimes = []
 
-    @util.cached_property
     def __song_key(self):
         return (self("~#disc"), self("~#track"),
             human(self("artistsort")),
@@ -108,7 +117,7 @@ class AudioFile(dict, ImageContainer):
 
     @util.cached_property
     def sort_key(self):
-        return [self.album_key, self.__song_key]
+        return [self.album_key, self.__song_key()]
 
     @staticmethod
     def sort_by_func(tag):
@@ -142,7 +151,6 @@ class AudioFile(dict, ImageContainer):
         pop = self.__dict__.pop
         pop("album_key", None)
         pop("sort_key", None)
-        pop("__song_key", None)
 
     def __delitem__(self, key):
         dict.__delitem__(self, key)
@@ -151,7 +159,6 @@ class AudioFile(dict, ImageContainer):
         pop = self.__dict__.pop
         pop("album_key", None)
         pop("sort_key", None)
-        pop("__song_key", None)
 
     @property
     def key(self):
@@ -203,6 +210,24 @@ class AudioFile(dict, ImageContainer):
 
         return filter(lambda s: s[:1] != "~", self.keys())
 
+    def prefixkeys(self, prefix):
+        """Returns a list of dict keys that either match prefix or start
+        with prefix + ':'.
+        """
+
+        l = []
+        for k in self:
+            if k.startswith(prefix):
+                if k == prefix or k.startswith(prefix + ":"):
+                    l.append(k)
+        return l
+
+    def _prefixvalue(self, tag):
+        return "\n".join(self.list_unique(sorted(self.prefixkeys(tag))))
+
+    def iterrealitems(self):
+        return ((k, v) for (k, v) in self.iteritems() if k[:1] != "~")
+
     def __call__(self, key, default=u"", connector=" - "):
         """Return a key, synthesizing it if necessary. A default value
         may be given (like dict.get); the default default is an empty
@@ -210,20 +235,22 @@ class AudioFile(dict, ImageContainer):
 
         If a tied tag ('a~b') is requested, the 'connector' keyword
         argument may be used to specify what it is tied with.
+        In case the tied tag contains numeric and file path tags, the result
+        will still be a unicode string.
 
-        For details on tied tags, see the documentation for util.tagsplit."""
+        For details on tied tags, see the documentation for util.tagsplit.
+        """
 
         if key[:1] == "~":
             key = key[1:]
             if "~" in key:
-                # FIXME: decode ~filename etc.
-                if not isinstance(default, basestring):
-                    return default
-                return connector.join(
-                    filter(None,
-                    map(lambda x: isinstance(x, basestring) and x or str(x),
-                    map(lambda x: (isinstance(x, float) and "%.2f" % x) or x,
-                    map(self.__call__, util.tagsplit("~" + key)))))) or default
+                real_key = "~" + key
+                values = []
+                for v in map(self.__call__, util.tagsplit(real_key)):
+                    v = decode_value(real_key, v)
+                    if v:
+                        values.append(v)
+                return connector.join(values) or default
             elif key == "#track":
                 try:
                     return int(self["tracknumber"].split("/")[0])
@@ -239,34 +266,42 @@ class AudioFile(dict, ImageContainer):
                 if length is None:
                     return default
                 else:
-                    return util.format_time(length)
+                    return util.format_time_display(length)
             elif key == "#rating":
                 return dict.get(self, "~" + key, config.RATINGS.default)
             elif key == "rating":
                 return util.format_rating(self("~#rating"))
-            elif key == "people" or key == "people:roles":
-                return (self._role_call(key, PEOPLE, "performer", True)
+            elif key == "people":
+                return "\n".join(self.list_unique(PEOPLE)) or default
+            elif key == "people:real":
+                # Issue 1034: Allow removal of V.A. if others exist.
+                unique = self.list_unique(PEOPLE)
+                # Order is important, for (unlikely case): multiple removals
+                for val in VARIOUS_ARTISTS_VALUES:
+                    if len(unique) > 1 and val in unique:
+                        unique.remove(val)
+                return "\n".join(unique) or default
+            elif key == "people:roles":
+                return (self._role_call("performer", PEOPLE)
                         or default)
-            elif key == "peoplesort" or key == "peoplesort:roles":
+            elif key == "peoplesort":
+                return ("\n".join(self.list_unique(PEOPLE_SORT)) or
+                        self("~people", default, connector))
+            elif key == "peoplesort:roles":
                 # Ignores non-sort tags if there are any sort tags (e.g. just
                 # returns "B" for {artist=A, performersort=B}).
                 # TODO: figure out the "correct" behavior for mixed sort tags
-                return (self._role_call(key, PEOPLE_SORT, "performersort",
-                                        True)
-                        or self("~" + key.replace("sort", ""),
-                                default, connector))
-            elif (key == "performers" or key == "performer" or
-                    key == "performers:roles" or key == "performer:roles"):
-                return (self._role_call(key,
-                                        prefixed("performer", self.keys()),
-                                        "performer")
-                        or default)
-            elif (key == "performerssort" or key == "performersort" or
-                  key == "performerssort:roles" or
-                  key == "performersort:roles"):
-                return (self._role_call(key,
-                                        prefixed("performersort", self.keys()),
-                                        "performersort")
+                return (self._role_call("performersort", PEOPLE_SORT)
+                        or self("~peoplesort", default, connector))
+            elif key in ("performers", "performer"):
+                return self._prefixvalue("performer") or default
+            elif key in ("performerssort", "performersort"):
+                return (self._prefixvalue("performersort") or
+                        self("~" + key[-4:], default, connector))
+            elif key in ("performers:roles", "performer:roles"):
+                return (self._role_call("performer") or default)
+            elif key in ("performerssort:roles", "performersort:roles"):
+                return (self._role_call("performersort")
                         or self("~" + key.replace("sort", ""), default,
                                 connector))
             elif key == "basename":
@@ -280,6 +315,23 @@ class AudioFile(dict, ImageContainer):
                     return URI.frompath(self["~filename"])
             elif key == "format":
                 return self.get("~format", self.format)
+            elif key == "codec":
+                codec = self.get("~codec")
+                if codec is None:
+                    return self("~format")
+                return codec
+            elif key == "encoding":
+                parts = filter(None,
+                               [self.get("~encoding"), self.get("encodedby")])
+                encoding = u"\n".join(parts)
+                return encoding or default
+            elif key == "bitrate":
+                return util.format_bitrate(self("~#bitrate"))
+            elif key == "#date":
+                date = self.get("date")
+                if date is None:
+                    return default
+                return util.date_key(date)
             elif key == "year":
                 return self.get("date", default)[:4]
             elif key == "#year":
@@ -350,8 +402,8 @@ class AudioFile(dict, ImageContainer):
             title = dict.get(self, "title")
             if title is None:
                 basename = self("~basename")
-                basename = basename.decode(const.FSCODING, "replace")
-                return "%s [%s]" % (basename, _("Unknown"))
+                return "%s [%s]" % (
+                    decode_value("~basename", basename), _("Unknown"))
             else:
                 return title
         elif key in SORT_TO_TAG:
@@ -361,41 +413,35 @@ class AudioFile(dict, ImageContainer):
                 key = SORT_TO_TAG[key]
         return dict.get(self, key, default)
 
-    def _roles(self, tag):
-        """Returns a defaultdict of name => [role, ...]."""
-        roles = collections.defaultdict(list)
-        for key in self.keys():
-            if key.startswith(tag + ":"):
-                role = util.title(key[1 + len(tag):])
+    def _role_call(self, role_tag, sub_keys=None):
+        role_tag_keys = self.prefixkeys(role_tag)
+
+        role_map = {}
+        for key in role_tag_keys:
+            if key != role_tag:
+                role = key.rsplit(":", 1)[-1]
                 for name in self.list(key):
-                    roles[name].append(role)
-        for name in self.list(tag):
-            roles[name]
-        return roles
+                    role_map.setdefault(name, []).append(role)
 
-    def _role_call(self, key, sub_keys, role_tag, use_pseudo_roles=False):
-        """Used to implement __call__ for a synthetic key with role support."""
-        names = []
-        names_seen = set()
-        for sub_key in sub_keys:
-            for value in self.list(sub_key):
-                if value not in names_seen:
-                    names_seen.add(value)
-                    names.append(value)
-
-        not_role_tag = lambda t: t != role_tag and t != '~' + role_tag
-        if key.endswith(":roles"):
-            roles = self._roles(role_tag)
-            if use_pseudo_roles:
-                for tag in filter(not_role_tag, sub_keys):
-                    for name in self.list(tag):
-                        if tag in TAG_ROLES:
-                            roles[name].append(TAG_ROLES[tag])
-                        else:
-                            roles[name]
-            return "\n".join(role_desc(n, roles[n]) for n in names)
+        if sub_keys is None:
+            names = self.list_unique(role_tag_keys)
         else:
-            return "\n".join(names)
+            names = self.list_unique(sub_keys)
+            for tag in sub_keys:
+                if tag in TAG_ROLES:
+                    for name in self.list(tag):
+                        role_map.setdefault(name, []).append(TAG_ROLES[tag])
+
+        descs = []
+        for name in names:
+            roles = role_map.get(name, [])
+            if not roles:
+                descs.append(name)
+            else:
+                roles = sorted(map(capitalize, roles))
+                descs.append("%s (%s)" % (name, ", ".join(roles)))
+
+        return "\n".join(descs)
 
     @property
     def lyric_filename(self):
@@ -420,13 +466,19 @@ class AudioFile(dict, ImageContainer):
 
     def comma(self, key):
         """Get all values of a tag, separated by commas. Synthetic
-        tags are supported, but will be slower. If the value is
-        numeric, that is returned rather than a list."""
+        tags are supported, but will be slower. All list items
+        will be unicode.
+
+        If the value is numeric, that is returned rather than a list.
+        """
 
         if "~" in key or key == "title":
             v = self(key, u"")
+            if key in FILESYSTEM_TAGS:
+                v = fsdecode(v)
         else:
             v = self.get(key, u"")
+
         if isinstance(v, (int, long, float)):
             return v
         else:
@@ -436,8 +488,12 @@ class AudioFile(dict, ImageContainer):
         """Get all values of a tag, as a list. Synthetic tags are supported,
         but will be slower. Numeric tags are not supported.
 
+        For file path keys the returned list might contain path items
+        (non-unicode).
+
         An empty synthetic tag cannot be distinguished from a non-existent
-        synthetic tag; both result in []."""
+        synthetic tag; both result in [].
+        """
 
         if "~" in key or key == "title":
             v = self(key, connector="\n")
@@ -451,14 +507,18 @@ class AudioFile(dict, ImageContainer):
 
     def list_separate(self, key, connector=" - "):
         """Similar to list, but will return a list of all combinations
-        for tied tags instead of one comma separated string"""
+        for tied tags instead of one comma separated string.
+
+        In case of tied tags the result will be unicode, otherwise
+        it returns the same as list()
+        """
+
         if key[:1] == "~" and "~" in key[1:]:
-            vals = \
-                filter(None,
-                map(lambda x: isinstance(x, basestring) and x or str(x),
-                map(lambda x: (isinstance(x, float) and "%.2f" % x) or x,
-                (self(tag) for tag in util.tagsplit(key)))))
-            vals = (val.split("\n") for val in vals)
+            vals = []
+            for v in map(self.__call__, util.tagsplit(key)):
+                v = decode_value(key, v)
+                if v:
+                    vals.append(v.split("\n"))
             r = [[]]
             for x in vals:
                 r = [i + [y] for y in x for i in r]
@@ -466,6 +526,22 @@ class AudioFile(dict, ImageContainer):
         else:
             return self.list(key)
 
+    def list_unique(self, keys):
+        """Returns a combined value of all values in keys; duplicate values
+        will be ignored.
+
+        Returns the same as list().
+        """
+
+        l = []
+        seen = set()
+        for k in keys:
+            for v in self.list(k):
+                if v not in seen:
+                    l.append(v)
+                    seen.add(v)
+        return l
+
     def as_lowercased(self):
         """Returns a new AudioFile with all keys lowercased / values merged.
 
@@ -501,31 +577,37 @@ class AudioFile(dict, ImageContainer):
         the file is not on a disk."""
         return os.path.ismount(self.get("~mountpoint", "/"))
 
+    def can_multiple_values(self, key=None):
+        """If no arguments are given, return a list of tags that can
+        have multiple values, or True if 'any' tags can.
+        """
+
+        return True
+
     def can_change(self, k=None):
         """See if this file supports changing the given tag. This may
-        be a limitation of the file type, or the file may not be
-        writable.
+        be a limitation of the file type or QL's design.
+
+        The writing code should handle all kinds of keys, so this is
+        just a suggestion.
 
         If no arguments are given, return a list of tags that can be
         changed, or True if 'any' tags can be changed (specific tags
         should be checked before adding)."""
 
         if k is None:
-            if os.access(self["~filename"], os.W_OK):
-                return True
-            else:
-                return []
+            return True
 
-        try:
-            if isinstance(k, unicode):
-                k = k.encode("ascii")
-            else:
-                k.decode("ascii")
-        except UnicodeError:
+        if not isascii(k):
             return False
 
-        return (k and "=" not in k and "~" not in k
-                and os.access(self["~filename"], os.W_OK))
+        if not k or "=" in k or "~" in k:
+            return False
+
+        return True
+
+    def is_writable(self):
+        return os.access(self["~filename"], os.W_OK)
 
     def rename(self, newname):
         """Rename a file. Errors are not handled. This shouldn't be used
@@ -588,6 +670,9 @@ class AudioFile(dict, ImageContainer):
             self["~filename"] = filename
         elif "~filename" not in self:
             raise ValueError("Unknown filename!")
+
+        assert is_fsnative(self["~filename"])
+
         if self.is_file:
             self["~filename"] = normalize_path(
                 self["~filename"], canonicalise=True)
@@ -601,7 +686,7 @@ class AudioFile(dict, ImageContainer):
                 if os.path.ismount(head):
                     self["~mountpoint"] = head
         else:
-            self["~mountpoint"] = "/"
+            self["~mountpoint"] = fsnative(u"/")
 
         # Fill in necessary values.
         self.setdefault("~#added", int(time.time()))
@@ -626,21 +711,27 @@ class AudioFile(dict, ImageContainer):
 
     def to_dump(self):
         """A string of 'key=value' lines, similar to vorbiscomment output."""
+
+        def encode_key(k):
+            return encode(k) if isinstance(k, unicode) else k
+
         s = []
         for k in self.keys():
-            k = str(k)
+            enc_key = encode_key(k)
+
             if isinstance(self[k], int) or isinstance(self[k], long):
-                s.append("%s=%d" % (k, self[k]))
+                s.append("%s=%d" % (enc_key, self[k]))
             elif isinstance(self[k], float):
-                s.append("%s=%f" % (k, self[k]))
+                s.append("%s=%f" % (enc_key, self[k]))
             else:
                 for v2 in self.list(k):
                     if isinstance(v2, str):
-                        s.append("%s=%s" % (k, v2))
+                        s.append("%s=%s" % (enc_key, v2))
                     else:
-                        s.append("%s=%s" % (k, encode(v2)))
+                        s.append("%s=%s" % (enc_key, encode(v2)))
         for k in (INTERN_NUM_DEFAULT - set(self.keys())):
-            s.append("%s=%d" % (k, self.get(k, 0)))
+            enc_key = encode_key(k)
+            s.append("%s=%d" % (enc_key, self.get(k, 0)))
         if "~#rating" not in self:
             s.append("~#rating=%f" % self("~#rating"))
         s.append("~format=%s" % self.format)
@@ -649,6 +740,15 @@ class AudioFile(dict, ImageContainer):
 
     def from_dump(self, text):
         """Parses the text created with to_dump and adds the found tags."""
+
+        def decode_key(key):
+            """str if ascii, otherwise decode using utf-8"""
+            try:
+                key.decode("ascii")
+            except ValueError:
+                return decode(key)
+            return key
+
         for line in text.split("\n"):
             if not line:
                 continue
@@ -666,7 +766,7 @@ class AudioFile(dict, ImageContainer):
                     except ValueError:
                         pass
             else:
-                self.add(key, val)
+                self.add(decode_key(key), decode(val))
 
     def change(self, key, old_value, new_value):
         """Change 'old_value' to 'new_value' for the given metadata key.
@@ -689,26 +789,24 @@ class AudioFile(dict, ImageContainer):
         else:
             self[key] += "\n" + value
 
-    def remove(self, key, value):
-        """Remove a value from the given key; if the value is not found,
-        remove all values for that key."""
+    def remove(self, key, value=None):
+        """Remove a value from the given key.
+
+        If value is None remove all values for that key, if it exists.
+        If the key or value is not found do nothing.
+        """
+
         if key not in self:
             return
-        elif self[key] == value:
-            del(self[key])
+        elif value is None or self[key] == value:
+            del self[key]
         else:
             try:
                 parts = self.list(key)
                 parts.remove(value)
-                self[key] = "\n".join(parts)
+                self[key] = u"\n".join(parts)
             except ValueError:
-                if key in self:
-                    del(self[key])
-
-    def find_cover(self):
-        """Return a file-like containing cover image data, or None if
-        no cover is available."""
-        return cover_plugins.acquire_cover_sync(self)
+                pass
 
     def replay_gain(self, profiles, pre_amp_gain=0, fallback_gain=0):
         """Return the computed Replay Gain scale factor.
@@ -743,7 +841,17 @@ class AudioFile(dict, ImageContainer):
         """Write metadata back to the file."""
         raise NotImplementedError
 
-    def __get_bookmarks(self):
+    @property
+    def bookmarks(self):
+        """Parse and return song position bookmarks, or set them.
+        Accessing this returns a copy, so song.bookmarks.append(...)
+        will not work; you need to do
+
+            marks = song.bookmarks
+            marks.append(...)
+            song.bookmarks = marks
+        """
+
         marks = []
         invalid = []
         for line in self.list("~bookmark"):
@@ -765,7 +873,8 @@ class AudioFile(dict, ImageContainer):
         marks.extend(invalid)
         return marks
 
-    def __set_bookmarks(self, marks):
+    @bookmarks.setter
+    def bookmarks(self, marks):
         result = []
         for time_, mark in marks:
             if time_ < 0:
@@ -777,15 +886,6 @@ class AudioFile(dict, ImageContainer):
         elif "~bookmark" in self:
             del(self["~bookmark"])
 
-    bookmarks = property(
-        __get_bookmarks, __set_bookmarks,
-        doc="""Parse and return song position bookmarks, or set them.
-        Accessing this returns a copy, so song.bookmarks.append(...)
-        will not work; you need to do
-           marks = song.bookmarks
-           marks.append(...)
-           song.bookmarks = marks""")
-
 
 # Looks like the real thing.
 DUMMY_SONG = AudioFile({
@@ -794,11 +894,3 @@ DUMMY_SONG = AudioFile({
     'title': 'First Track', 'tracknumber': 1,
     'date': '2010-12-31',
 })
-
-
-def role_desc(name, roles):
-    return name if not roles else "%s (%s)" % (name, ", ".join(sorted(roles)))
-
-
-def prefixed(prefix, strings):
-    return filter(lambda s: s == prefix or s.startswith(prefix + ":"), strings)
