@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2004-2013 Joe Wreschnig, Michael Urman, Niklas Janlert,
 #                     Steven Robertson, Nick Boultbee
 #
@@ -10,14 +11,12 @@ import mutagen.id3
 from quodlibet import config, const, print_d
 from quodlibet import util
 from quodlibet.config import RATINGS
-from quodlibet.formats._audio import AudioFile
-from quodlibet.formats._image import EmbeddedImage, APICType
 from quodlibet.util.massagers import LanguageMassager
 from quodlibet.util.path import get_temp_cover_file
+from quodlibet.util.string import isascii
 
-
-def isascii(s):
-    return (len(s) == 0) or (ord(max(s)) < 128)
+from ._audio import AudioFile
+from ._image import EmbeddedImage, APICType
 
 
 def encoding_for(s):
@@ -35,14 +34,10 @@ class ID3hack(mutagen.id3.ID3):
         else:
             self[tag.HashKey] = tag
 
-
-class ID3bug(mutagen.id3.ID3):
-    def load(self, *args, **kwargs):
-        # Work around Mutagen bug by throwing away unknown 2.3 frames on load
-        # see http://code.google.com/p/mutagen/issues/detail?id=97
-        super(ID3bug, self).load(*args, **kwargs)
-        if self.version == (2, 3, 0) and mutagen.version < (1, 21):
-            del self.unknown_frames[:]
+RG_KEYS = [
+    "replaygain_track_peak", "replaygain_track_gain",
+    "replaygain_album_peak", "replaygain_album_gain",
+]
 
 
 # ID3 is absolutely the worst thing ever.
@@ -113,6 +108,7 @@ class ID3File(AudioFile):
     def __init__(self, filename):
         audio = self.Kind(filename, ID3=ID3hack)
         tag = audio.tags or mutagen.id3.ID3()
+        self._parse_info(audio.info)
 
         for frame in tag.values():
             if frame.FrameID == "APIC" and len(frame.data):
@@ -123,7 +119,7 @@ class ID3File(AudioFile):
                 continue
             elif frame.FrameID == "TLEN":
                 try:
-                    length = +frame // 1000
+                    length = +frame / 1000.0
                 except ValueError:
                     continue
                 # ignore TLEN <= 0 [issue 222]
@@ -211,21 +207,19 @@ class ID3File(AudioFile):
             for frame in tag.getall('TXXX:DATE'):
                 self["date"] = "\n".join(map(unicode, frame.text))
 
-        # Read TXXX replaygain in case we don't have any (from RVA2)
-        for k in ["track_peak", "track_gain", "album_peak", "album_gain"]:
-            k = "replaygain_" + k
-            if k not in self:
-                for frame in tag.getall("TXXX:" + k):
-                    self[k] = "\n".join(map(unicode, frame.text))
-
-        self.setdefault("~#length", int(audio.info.length))
-        try:
-            self.setdefault("~#bitrate", int(audio.info.bitrate / 1000))
-        except AttributeError:
-            pass
+        # Read TXXX replaygain and replace previously read values from RVA2
+        for frame in tag.getall("TXXX"):
+            k = frame.desc.lower()
+            if k in RG_KEYS:
+                self[str(k)] = "\n".join(map(unicode, frame.text))
 
         self.sanitize(filename)
 
+    def _parse_info(self, info):
+        """Optionally implement in subclasses"""
+
+        pass
+
     def __validate_name(self, k):
         """Returns a ascii string or None if the key isn't supported"""
         if isinstance(k, unicode):
@@ -273,7 +267,7 @@ class ID3File(AudioFile):
 
     def write(self):
         try:
-            tag = ID3bug(self['~filename'])
+            tag = mutagen.id3.ID3(self['~filename'])
         except mutagen.id3.error:
             tag = mutagen.id3.ID3()
 
@@ -308,10 +302,8 @@ class ID3File(AudioFile):
             else:
                 tag.add(Kind(encoding=enc, text=text))
 
-        dontwrite = ["genre", "comment", "replaygain_album_peak",
-                     "replaygain_track_peak", "replaygain_album_gain",
-                     "replaygain_track_gain", "musicbrainz_trackid",
-                     ] + self.TXXX_MAP.values()
+        dontwrite = ["genre", "comment", "musicbrainz_trackid"] \
+            + RG_KEYS + self.TXXX_MAP.values()
 
         if "musicbrainz_trackid" in self.realkeys():
             f = mutagen.id3.UFID(owner="http://musicbrainz.org",
@@ -364,14 +356,13 @@ class ID3File(AudioFile):
             tag.add(mutagen.id3.COMM(encoding=enc, text=t, desc=u"",
                                      lang="\x00\x00\x00"))
 
-        # Delete old foobar replaygain and write new one
-        for k in ["track_peak", "track_gain", "album_peak", "album_gain"]:
-            k = "replaygain_" + k
-            # Delete Foobar droppings.
-            try:
-                del(tag["TXXX:" + k])
-            except KeyError:
-                pass
+        # Delete old foobar replaygain ..
+        for frame in tag.getall("TXXX"):
+            if frame.desc.lower() in RG_KEYS:
+                del tag[frame.HashKey]
+
+        # .. write new one
+        for k in RG_KEYS:
             # Add new ones
             if k in self:
                 value = self[k]
@@ -395,7 +386,7 @@ class ID3File(AudioFile):
                     peak = float(self["replaygain_%s_peak" % k])
                 except (ValueError, KeyError):
                     peak = 0
-                # http://code.google.com/p/quodlibet/issues/detail?id=1027
+                # https://github.com/quodlibet/quodlibet/issues/1027
                 peak = max(min(1.9, peak), 0)
                 gain = max(min(63.9, gain), -64)
                 f = mutagen.id3.RVA2(desc=k, channel=1, gain=gain, peak=peak)
@@ -407,6 +398,9 @@ class ID3File(AudioFile):
             except KeyError:
                 pass
         for key in self.PAM_XXXT:
+            if key in self.SDI:
+                # we already write it back using non-TXXX frames
+                continue
             if key in self:
                 value = self[key]
                 f = mutagen.id3.TXXX(encoding=encoding_for(value),
@@ -485,7 +479,7 @@ class ID3File(AudioFile):
         try:
             tag = mutagen.id3.ID3(self["~filename"])
         except Exception:
-            return
+            tag = mutagen.id3.ID3()
 
         try:
             data = image.file.read()
@@ -497,6 +491,6 @@ class ID3File(AudioFile):
             encoding=3, mime=image.mime_type, type=APICType.COVER_FRONT,
             desc=u"", data=data)
         tag.add(frame)
-        tag.save()
+        tag.save(self["~filename"])
 
         self.has_images = True
