@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2005 Alexey Bobyakov <claymore.ws@gmail.com>, Joe Wreschnig
 # Copyright 2006 Lukas Lalinsky
 #
@@ -5,23 +6,17 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from quodlibet.formats._audio import AudioFile
-from quodlibet.formats._image import EmbeddedImage, APICType
+from mutagen.mp4 import MP4, MP4Cover
+
 from quodlibet.util.path import get_temp_cover_file
 from quodlibet.util.string import decode
 
-
-extensions = ['.mp4', '.m4a', '.m4v']
-
-try:
-    from mutagen.mp4 import MP4, MP4Cover
-except ImportError:
-    extensions = []
+from ._audio import AudioFile
+from ._image import EmbeddedImage
 
 
 class MP4File(AudioFile):
-    multiple_values = False
-    format = "MPEG-4 AAC"
+    format = "MPEG-4"
     mimes = ["audio/mp4", "audio/x-m4a", "audio/mpeg4", "audio/aac"]
 
     __translate = {
@@ -35,7 +30,7 @@ class MP4File(AudioFile):
         "\xa9grp": "grouping",
         "\xa9gen": "genre",
         "tmpo": "bpm",
-        "\xa9too": "encodedby",
+        "\xa9too": "encodedby",  # FIXME: \xa9enc should be encodedby
         "cprt": "copyright",
         "soal": "albumsort",
         "soaa": "albumartistsort",
@@ -66,8 +61,10 @@ class MP4File(AudioFile):
 
     def __init__(self, filename):
         audio = MP4(filename)
-        self["~#length"] = int(audio.info.length)
+        self["~codec"] = getattr(audio.info, "codec_description", u"AAC")
+        self["~#length"] = audio.info.length
         self["~#bitrate"] = int(audio.info.bitrate / 1000)
+
         for key, values in audio.items():
             if key in self.__tupletranslate:
                 name = self.__tupletranslate[key]
@@ -117,6 +114,11 @@ class MP4File(AudioFile):
         audio.save()
         self.sanitize()
 
+    def can_multiple_values(self, key=None):
+        if key is None:
+            return []
+        return False
+
     def can_change(self, key=None):
         OK = self.__rtranslate.keys() + self.__rtupletranslate.keys()
         if key is None:
@@ -207,3 +209,4 @@ class MP4File(AudioFile):
 
 info = MP4File
 types = [MP4File]
+extensions = ['.mp4', '.m4a', '.m4v', '.3gp', '.3g2', '.3gp2']
