@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2004-2011 Joe Wreschnig, Michael Urman, Steven Robertson,
 #           2011-2014 Christoph Reiter
 #
@@ -14,15 +15,15 @@ except ValueError, e:
 
 from gi.repository import Gst, GLib, GstPbutils
 
-import threading
-
 from quodlibet import const
 from quodlibet import config
+from quodlibet import util
+from quodlibet import app
 
-from quodlibet.util import fver, sanitize_tags
+from quodlibet.util import fver, sanitize_tags, MainRunner, MainRunnerError, \
+    MainRunnerAbortedError, MainRunnerTimeoutError
 from quodlibet.player import PlayerError
 from quodlibet.player._base import BasePlayer
-from quodlibet.qltk.msg import ErrorMessage
 from quodlibet.qltk.notif import Task
 
 from .util import (parse_gstreamer_taglist, TagListWrapper, iter_to_list,
@@ -141,44 +142,68 @@ class BufferingWrapper(object):
             self.__bus_id = None
 
         self.__set_inhibit_play(False)
+        self.bin = None
+
+
+def sink_has_external_state(sink):
+    return sink.get_factory().get_name() == "pulsesink"
+
+
+def sink_state_is_valid(sink):
+    if not sink_has_external_state(sink):
+        return True
+
+    # pulsesink volume is only valid in PAUSED/PLAYING
+    # https://bugzilla.gnome.org/show_bug.cgi?id=748577
+    current_state = sink.get_state(0)[1]
+    return current_state >= Gst.State.PAUSED
 
 
 class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
-    __gproperties__ = BasePlayer._gproperties_
-    __gsignals__ = BasePlayer._gsignals_
-
-    _paused = True
-    _in_gapless_transition = False
-    _last_position = 0
-
-    bin = None
-    _vol_element = None
-    _use_eq = False
-    _eq_element = None
-
-    __atf_id = None
-    __bus_id = None
-
-    __info_buffer = None
 
     def PlayerPreferences(self):
         return GstPlayerPreferences(self, const.DEBUG)
 
     def __init__(self, librarian=None):
         GStreamerPluginHandler.__init__(self)
-        super(GStreamerPlayer, self).__init__()
-        self.version_info = "GStreamer: %s" % fver(Gst.version())
+        BasePlayer.__init__(self)
+
         self._librarian = librarian
+
+        self.version_info = "GStreamer: %s" % fver(Gst.version())
         self._pipeline_desc = None
-        librarian.connect("changed", self.__songs_changed)
+
+        self._volume = 1.0
+        self._paused = True
+        self._seekable = False
+        self._mute = False
+
+        self._in_gapless_transition = False
         self._active_seeks = []
+        self._active_error = False
+        self._last_position = 0
+
+        self.bin = None
+        self._int_vol_element = None
+        self._ext_vol_element = None
+        self._ext_mute_element = None
+        self._use_eq = False
+        self._eq_element = None
+        self.__info_buffer = None
+
+        self._lib_id = librarian.connect("changed", self.__songs_changed)
+        self.__atf_id = None
+        self.__bus_id = None
+        self._runner = MainRunner()
 
     def __songs_changed(self, librarian, songs):
         # replaygain values might have changed, recalc volume
         if self.song and self.song in songs:
-            self.volume = self.volume
+            self._reset_replaygain()
 
-    def destroy(self):
+    def _destroy(self):
+        self._librarian.disconnect(self._lib_id)
+        self._runner.abort()
         self.__destroy_pipeline()
 
     @property
@@ -188,6 +213,13 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             name += " (%s)" % self._pipeline_desc
         return name
 
+    @property
+    def has_external_volume(self):
+        ext = self._ext_vol_element
+        if ext is None or not sink_has_external_state(ext):
+            return False
+        return True
+
     def _set_buffer_duration(self, duration):
         """Set the stream buffer duration in msecs"""
 
@@ -209,12 +241,28 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         else:
             print_e("No active pipeline.")
 
+    def _refresh_seekable(self):
+        query = Gst.Query.new_seeking(Gst.Format.TIME)
+        if self.bin and self.bin.query(query):
+            seekable = query.parse_seeking()[1]
+        elif self.song is None:
+            seekable = False
+        else:
+            seekable = True
+
+        if seekable != self._seekable:
+            self._seekable = seekable
+            self.notify("seekable")
+
     def __init_pipeline(self):
         """Creates a gstreamer pipeline. Returns True on success."""
 
         if self.bin:
             return True
 
+        # reset error state
+        self.error = False
+
         pipeline = config.get("player", "gst_pipeline")
         try:
             pipeline, self._pipeline_desc = GStreamerSink(pipeline)
@@ -243,8 +291,8 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         # playbin2 has started to control the volume through pulseaudio,
         # which means the volume property can change without us noticing.
         # Use our own volume element for now until this works with PA.
-        self._vol_element = Gst.ElementFactory.make('volume', None)
-        pipeline.insert(0, self._vol_element)
+        self._int_vol_element = Gst.ElementFactory.make('volume', None)
+        pipeline.insert(0, self._int_vol_element)
 
         # Get all plugin elements and append audio converters.
         # playbin already includes one at the end
@@ -279,6 +327,40 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self._error(PIPELINE_ERROR)
             return False
 
+        # see if the sink provides a volume property, if yes, use it
+        sink_element = pipeline[-1]
+        if isinstance(sink_element, Gst.Bin):
+            sink_element = iter_to_list(sink_element.iterate_recurse)[-1]
+
+        self._ext_vol_element = None
+        if hasattr(sink_element.props, "volume"):
+            self._ext_vol_element = sink_element
+
+            # In case we use the sink volume directly we can increase buffering
+            # without affecting the volume change delay too much and safe some
+            # CPU time... (2x default for now).
+            if hasattr(sink_element.props, "buffer_time"):
+                sink_element.set_property("buffer-time", 400000)
+
+            def ext_volume_notify(*args):
+                # gets called from a thread
+                GLib.idle_add(self.notify, "volume")
+
+            self._ext_vol_element.connect("notify::volume", ext_volume_notify)
+
+        self._ext_mute_element = None
+        if hasattr(sink_element.props, "mute") and \
+                sink_element.get_factory().get_name() != "directsoundsink":
+            # directsoundsink has a mute property but it doesn't work
+            # https://bugzilla.gnome.org/show_bug.cgi?id=755106
+            self._ext_mute_element = sink_element
+
+            def mute_notify(*args):
+                # gets called from a thread
+                GLib.idle_add(self.notify, "mute")
+
+            self._ext_mute_element.connect("notify::mute", mute_notify)
+
         # Make the sink of the first element the sink of the bin
         gpad = Gst.GhostPad.new('sink', pipeline[0].get_static_pad('sink'))
         bufbin.add_pad(gpad)
@@ -334,8 +416,13 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                     break
         self.bin.connect("source-setup", source_setup)
 
-        # ReplayGain information gets lost when destroying
-        self.volume = self.volume
+        if self.has_external_volume:
+            # ReplayGain information gets lost when destroying
+            self._reset_replaygain()
+        else:
+            # Restore volume/ReplayGain and mute state
+            self.volume = self._volume
+            self.mute = self._mute
 
         if self.song:
             self.bin.set_property('uri', self.song("~uri"))
@@ -349,11 +436,11 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             bus = self.bin.get_bus()
             bus.disconnect(self.__bus_id)
             bus.remove_signal_watch()
-            self.__bus_id = False
+            self.__bus_id = None
 
         if self.__atf_id:
             self.bin.disconnect(self.__atf_id)
-            self.__atf_id = False
+            self.__atf_id = None
 
         if self.bin:
             self.bin.set_state(Gst.State.NULL)
@@ -366,7 +453,9 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         self._last_position = 0
         self._active_seeks = []
 
-        self._vol_element = None
+        self._ext_vol_element = None
+        self._int_vol_element = None
+        self._ext_mute_element = None
         self._eq_element = None
 
     def _rebuild_pipeline(self):
@@ -404,7 +493,18 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                 # can contain paths, so not sure if utf-8 in all cases
                 details = debug_info.decode("utf-8", errors="replace")
             self._error(PlayerError(message, details))
+        elif message.type == Gst.MessageType.STATE_CHANGED:
+            # pulsesink doesn't notify a volume change on startup
+            # and the volume is only valid in > paused states.
+            if message.src is self._ext_vol_element:
+                self.notify("volume")
+            if message.src is self._ext_mute_element:
+                self.notify("mute")
 
+            if message.src is self.bin.bin:
+                new_state = message.parse_state_changed()[1]
+                if new_state >= Gst.State.PAUSED:
+                    self._refresh_seekable()
         elif message.type == Gst.MessageType.STREAM_START:
             if self._in_gapless_transition:
                 print_d("Stream changed")
@@ -419,8 +519,24 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
             if message_name == "missing-plugin":
                 self.__handle_missing_plugin(message)
-
-        return True
+        elif message.type == Gst.MessageType.CLOCK_LOST:
+            print_d("Clock lost")
+            self.bin.set_state(Gst.State.PAUSED)
+            self.bin.set_state(Gst.State.PLAYING)
+        elif message.type == Gst.MessageType.LATENCY:
+            print_d("Recalculate latency")
+            self.bin.recalculate_latency()
+        elif message.type == Gst.MessageType.REQUEST_STATE:
+            state = message.parse_request_state()
+            print_d("State requested: %s" % Gst.Element.state_get_name(state))
+            self.bin.set_state(state)
+        elif message.type == Gst.MessageType.DURATION_CHANGED:
+            if self.song.fill_length:
+                ok, p = self.bin.query_duration(Gst.Format.TIME)
+                if ok:
+                    p /= float(Gst.SECOND)
+                    self.song["~#length"] = p
+                    librarian.changed([self.song])
 
     def __handle_missing_plugin(self, message):
         get_installer_detail = \
@@ -443,6 +559,26 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             Gst.update_registry()
 
         context = GstPbutils.InstallPluginsContext.new()
+
+        # new in 1.6
+        if hasattr(context, "set_desktop_id"):
+            from gi.repository import Gtk
+            context.set_desktop_id(app.id)
+
+        # new in 1.6
+        if hasattr(context, "set_startup_notification_id"):
+            current_time = Gtk.get_current_event_time()
+            context.set_startup_notification_id("_TIME%d" % current_time)
+
+        gdk_window = app.window.get_window()
+        if gdk_window:
+            try:
+                xid = gdk_window.get_xid()
+            except AttributeError:  # non X11
+                pass
+            else:
+                context.set_xid(xid)
+
         res = GstPbutils.install_plugins_async(
             [details], context, install_done_cb, None)
         print_d("Gstreamer plugin install result: %r" % res)
@@ -451,58 +587,117 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                 GstPbutils.InstallPluginsReturn.INTERNAL_FAILURE):
             self._error(PlayerError(title, error_details))
 
-    def __about_to_finish(self, pipeline):
-        print_d("About to finish")
+    def __about_to_finish_sync(self):
+        """Returns a tuple (ok, next_song). ok is True if the next song
+        should be set.
+        """
+
+        print_d("About to finish (sync)")
 
         # Chained oggs falsely trigger a gapless transition.
         # At least for radio streams we can safely ignore it because
         # transitions don't occur there.
-        # https://code.google.com/p/quodlibet/issues/detail?id=1454
+        # https://github.com/quodlibet/quodlibet/issues/1454
         # https://bugzilla.gnome.org/show_bug.cgi?id=695474
         if self.song.multisong:
             print_d("multisong: ignore about to finish")
-            return
+            return (False, None)
 
         if config.getboolean("player", "gst_disable_gapless"):
             print_d("Gapless disabled")
-            return
+            return (False, None)
 
         # this can trigger twice, see issue 987
         if self._in_gapless_transition:
-            return
+            return (False, None)
         self._in_gapless_transition = True
 
-        def change_in_main_loop(event, source):
-            source.next_ended()
-            event.set()
+        print_d("Select next song in mainloop..")
+        self._source.next_ended()
+        print_d("..done.")
 
-        # push in the main loop and wait for it to finish
-        event = threading.Event()
-        GLib.idle_add(change_in_main_loop, event, self._source,
-                         priority=GLib.PRIORITY_HIGH)
-        event.wait()
+        return (True, self._source.current)
 
-        song = self._source.current
-        bin = self.bin
+    def __about_to_finish(self, playbin):
+        print_d("About to finish (async)")
 
-        if song and bin:
-            bin.set_property('uri', song("~uri"))
+        try:
+            ok, song = self._runner.call(self.__about_to_finish_sync,
+                                         priority=GLib.PRIORITY_HIGH,
+                                         timeout=0.5)
+        except MainRunnerTimeoutError as e:
+            # Due to some locks being held during this signal we can get
+            # into a deadlock when a seek or state change event happens
+            # in the mainloop before our function gets scheduled.
+            # In this case abort and do nothing, which results
+            # in a non-gapless transition.
+            print_d("About to finish (async): %s" % e)
+            return
+        except MainRunnerAbortedError:
+            print_d("About to finish (async): %s" % e)
+            return
+        except MainRunnerError:
+            util.print_exc()
+            return
+
+        if ok:
+            print_d("About to finish (async): setting uri")
+            uri = song("~uri") if song is not None else None
+            playbin.set_property('uri', uri)
+        print_d("About to finish (async): done")
 
     def stop(self):
         super(GStreamerPlayer, self).stop()
         self.__destroy_pipeline()
 
+    def do_get_property(self, property):
+        if property.name == 'volume':
+            if self._ext_vol_element is not None and \
+                    sink_has_external_state(self._ext_vol_element) and \
+                    sink_state_is_valid(self._ext_vol_element):
+                # never read back the volume if we don't have to, e.g.
+                # directsoundsink maps volume to an int which makes UI
+                # sliders jump if we read the value back
+                self._volume = self._ext_vol_element.get_property("volume")
+            return self._volume
+        elif property.name == "mute":
+            if self._ext_mute_element is not None and \
+                    sink_has_external_state(self._ext_mute_element) and \
+                    sink_state_is_valid(self._ext_mute_element):
+                self._mute = self._ext_mute_element.get_property("mute")
+            return self._mute
+        elif property.name == "seekable":
+            return self._seekable
+        else:
+            raise AttributeError
+
+    def _reset_replaygain(self):
+        if not self.bin:
+            return
+
+        v = 1.0 if self._ext_vol_element is not None else self._volume
+        v = self.calc_replaygain_volume(v)
+        v = min(10.0, max(0.0, v))
+        self._int_vol_element.set_property('volume', v)
+
     def do_set_property(self, property, v):
         if property.name == 'volume':
             self._volume = v
-            if self.song and config.getboolean("player", "replaygain"):
-                profiles = filter(None, self.replaygain_profiles)[0]
-                fb_gain = config.getfloat("player", "fallback_gain")
-                pa_gain = config.getfloat("player", "pre_amp_gain")
-                scale = self.song.replay_gain(profiles, pa_gain, fb_gain)
-                v = min(10.0, max(0.0, v * scale)) # volume supports 0..10
-            if self.bin:
-                self._vol_element.set_property('volume', v)
+            if self._ext_vol_element:
+                v = min(10.0, max(0.0, v))
+                self._ext_vol_element.set_property("volume", v)
+            else:
+                v = self.calc_replaygain_volume(v)
+                if self.bin:
+                    v = min(10.0, max(0.0, v))
+                    self._int_vol_element.set_property('volume', v)
+        elif property.name == 'mute':
+            self._mute = v
+            if self._ext_mute_element is not None:
+                self._ext_mute_element.set_property("mute", v)
+            else:
+                if self.bin:
+                    self._int_vol_element.set_property("mute", v)
         else:
             raise AttributeError
 
@@ -525,47 +720,41 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                 self._last_position = p
         return p
 
-    def _set_paused(self, paused):
+    @property
+    def paused(self):
+        return self._paused
+
+    @paused.setter
+    def paused(self, paused):
         if paused == self._paused:
             return
-        self._paused = paused
 
-        if not self.song:
-            if paused:
-                # Something wants us to pause between songs, or when
-                # we've got no song playing (probably StopAfterMenu).
-                self.emit('paused')
-                self.__destroy_pipeline()
+        self._paused = paused
+        self.emit((paused and 'paused') or 'unpaused')
+        # in case a signal handler changed the paused state, abort this
+        if self._paused != paused:
             return
 
         if paused:
             if self.bin:
-                if self.song.is_file:
-                    # fast path
+                if not self.song:
+                    # Something wants us to pause between songs, or when
+                    # we've got no song playing (probably StopAfterMenu).
+                    self.__destroy_pipeline()
+                elif self.seekable:
                     self.bin.set_state(Gst.State.PAUSED)
                 else:
-                    # seekable streams (seem to) have a duration >= 0
-                    ok, d = self.bin.query_duration(Gst.Format.TIME)
-                    if not ok:
-                        d = -1
-
-                    if d >= 0:
-                        self.bin.set_state(Gst.State.PAUSED)
-                    else:
-                        # destroy so that we rebuffer on resume
+                    q = Gst.Query.new_buffering(Gst.Format.DEFAULT)
+                    if self.bin.query(q):
+                        # destroy so that we rebuffer on resume i.e. we don't
+                        # want to continue unseekable streams from where we
+                        # paused but from where we unpaused.
                         self.__destroy_pipeline()
+                    else:
+                        self.bin.set_state(Gst.State.PAUSED)
         else:
-            if self.bin:
+            if self.song and self.__init_pipeline():
                 self.bin.set_state(Gst.State.PLAYING)
-            else:
-                if self.__init_pipeline():
-                    self.bin.set_state(Gst.State.PLAYING)
-
-        self.emit((paused and 'paused') or 'unpaused')
-
-    def _get_paused(self):
-        return self._paused
-    paused = property(_get_paused, _set_paused)
 
     def _error(self, player_error):
         """Destroy the pipeline and set the error state.
@@ -574,8 +763,9 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         """
 
         # prevent recursive errors
-        if self.error:
+        if self._active_error:
             return
+        self._active_error = True
 
         self.__destroy_pipeline()
         self.error = True
@@ -583,6 +773,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
         print_w(unicode(player_error))
         self.emit('error', self.song, player_error)
+        self._active_error = False
 
     def seek(self, pos):
         """Seek to a position in the song, in milliseconds."""
@@ -614,14 +805,14 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                 # event here and emit in the bus message callback.
                 self._active_seeks.append((self.song, pos))
 
-    def _end(self, stopped):
+    def _end(self, stopped, next_song=None):
         print_d("End song")
         song, info = self.song, self.info
 
         # set the new volume before the signals to avoid delays
         if self._in_gapless_transition:
             self.song = self._source.current
-            self.volume = self.volume
+            self._reset_replaygain()
 
         # We need to set self.song to None before calling our signal
         # handlers. Otherwise, if they try to end the song they're given
@@ -631,17 +822,15 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self.emit('song-ended', info, stopped)
         self.emit('song-ended', song, stopped)
 
-        # reset error state
-        self.error = False
+        current = self._source.current if next_song is None else next_song
 
         # Then, set up the next song.
-        self.song = self.info = self._source.current
+        self.song = self.info = current
         self.emit('song-started', self.song)
 
         print_d("Next song")
         if self.song is not None:
             if not self._in_gapless_transition:
-                self.volume = self.volume
                 # Due to extensive problems with playbin2, we destroy the
                 # entire pipeline and recreate it each time we're not in
                 # a gapless transition.
@@ -660,6 +849,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self.paused = True
 
         self._in_gapless_transition = False
+        self._refresh_seekable()
 
     def __tag(self, tags, librarian):
         if self.song and self.song.multisong:
@@ -751,12 +941,4 @@ def init(librarian):
     if Gst.debug_get_default_threshold() == Gst.DebugLevel.NONE:
         Gst.debug_set_default_threshold(Gst.DebugLevel.ERROR)
 
-    if Gst.Element.make_from_uri(
-        Gst.URIType.SRC,
-        "file:///fake/path/for/gst", ""):
-        return GStreamerPlayer(librarian)
-    else:
-        raise PlayerError(
-            _("Unable to open input files"),
-            _("GStreamer has no element to handle reading files. Check "
-              "your GStreamer installation settings."))
+    return GStreamerPlayer(librarian)
