@@ -45,8 +45,6 @@ class XineHandle(object):
 
 class XinePlaylistPlayer(BasePlayer):
     """Xine playlist player."""
-    __gproperties__ = BasePlayer._gproperties_
-    __gsignals__ = BasePlayer._gsignals_
 
     _paused = True
 
@@ -56,6 +54,7 @@ class XinePlaylistPlayer(BasePlayer):
         super(XinePlaylistPlayer, self).__init__()
         self.name = "xine"
         self.version_info = "xine-lib: " + xine_get_version_string()
+        self._volume = 1.0
         self._handle = XineHandle()
         self._supports_gapless = xine_check_version(1, 1, 1) == 1
         self._event_queue = None
@@ -152,18 +151,30 @@ class XinePlaylistPlayer(BasePlayer):
                 GLib.idle_add(self._error, PlayerError(message))
         return True
 
+    def do_get_property(self, property):
+        if property.name == 'volume':
+            return self._volume
+        elif property.name == 'seekable':
+            if self.song is None:
+                return False
+            return True
+        elif property.name == 'mute':
+            if not self._destroyed:
+                return xine_get_param(self._stream, XINE_PARAM_AUDIO_AMP_MUTE)
+            return False
+        else:
+            raise AttributeError
+
     def do_set_property(self, property, v):
         if property.name == 'volume':
             self._volume = v
-            if self.song and config.getboolean("player", "replaygain"):
-                profiles = filter(None, self.replaygain_profiles)[0]
-                fb_gain = config.getfloat("player", "fallback_gain")
-                pa_gain = config.getfloat("player", "pre_amp_gain")
-                scale = self.song.replay_gain(profiles, pa_gain, fb_gain)
-                v = max(0.0, v * scale)
+            v = self.calc_replaygain_volume(v)
             v = min(100, int(v * 100))
             if not self._destroyed:
                 xine_set_param(self._stream, XINE_PARAM_AUDIO_AMP_LEVEL, v)
+        elif property.name == 'mute':
+            if not self._destroyed:
+                xine_set_param(self._stream, XINE_PARAM_AUDIO_AMP_MUTE, v)
         else:
             raise AttributeError
 
@@ -264,6 +275,9 @@ class XinePlaylistPlayer(BasePlayer):
             self.paused = True
             xine_stop(self._stream)
 
+        # seekable might change if we change to None, so notify just in case
+        self.notify("seekable")
+
     def setup(self, playlist, song, seek_pos):
         super(XinePlaylistPlayer, self).setup(playlist, song, seek_pos)
         # xine's declining to seek so soon after startup; try again in 100ms
