@@ -8,7 +8,8 @@
 from gi.repository import Gtk
 from quodlibet.qltk import get_top_parent, get_menu_item_top_parent
 from quodlibet.qltk.msg import WarningMessage
-from quodlibet.qltk.x import SeparatorMenuItem, Button
+from quodlibet.qltk.x import SeparatorMenuItem
+from quodlibet.qltk import Icons
 from quodlibet.util import print_exc
 from quodlibet.util.dprint import print_d, print_e
 from quodlibet.plugins import PluginHandler, PluginManager
@@ -32,10 +33,9 @@ class ConfirmMultiPlaylistInvoke(WarningMessage):
             title, "",
             buttons=Gtk.ButtonsType.NONE)
 
-        self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        delete_button = Button(_("_Run Plugin"), Gtk.STOCK_EXECUTE)
-        delete_button.show()
-        self.add_action_widget(delete_button, self.RESPONSE_INVOKE)
+        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
+        self.add_icon_button(
+            _("_Run Plugin"), Icons.SYSTEM_RUN, self.RESPONSE_INVOKE)
         self.set_default_response(Gtk.ResponseType.CANCEL)
 
     @classmethod
