@@ -156,14 +156,38 @@ def _popup_menu_at_widget(menu, widget, button, time, under):
     return True
 
 
+def _ensure_menu_attached(menu, widget):
+    assert widget is not None
+
+    # Workaround the menu inheriting the wrong colors with the Ubuntu 12.04
+    # default themes. Attaching to the parent kinda works... submenus still
+    # have the wrong color.
+    if isinstance(widget, Gtk.Button):
+        widget = widget.get_parent() or widget
+
+    attached_widget = menu.get_attach_widget()
+    if attached_widget is widget:
+        return
+    if attached_widget is not None:
+        menu.detach()
+    menu.attach_to_widget(widget, None)
+
+
 def popup_menu_under_widget(menu, widget, button, time):
+    _ensure_menu_attached(menu, widget)
     return _popup_menu_at_widget(menu, widget, button, time, True)
 
 
 def popup_menu_above_widget(menu, widget, button, time):
+    _ensure_menu_attached(menu, widget)
     return _popup_menu_at_widget(menu, widget, button, time, False)
 
 
+def popup_menu_at_widget(menu, widget, button, time):
+    _ensure_menu_attached(menu, widget)
+    menu_popup(menu, None, None, None, None, button, time)
+
+
 def add_fake_accel(widget, accel):
     """Accelerators are only for window menus and global keyboard shortcuts.
 
@@ -345,7 +369,9 @@ def add_signal_watch(signal_action):
 # Legacy plugin/code support.
 from quodlibet.qltk.msg import *
 from quodlibet.qltk.x import *
+from quodlibet.qltk.icons import Icons
 from quodlibet.qltk.window import Window, UniqueWindow
 
 Window
 UniqueWindow
+Icons
