@@ -6,6 +6,7 @@
 # published by the Free Software Foundation
 
 import os
+import platform
 
 from gi.repository import Gtk, GLib, GdkPixbuf
 import mutagen
@@ -56,12 +57,14 @@ class AboutDialog(Gtk.AboutDialog):
         text.append("")
         if player:
             text.append(_("Audio device: %s") % player.name)
+        text.append("Python: %s" % platform.python_version())
         text.append("Mutagen: %s" % fver(mutagen.version))
         text.append("GTK+: %s" % fver(gtk_version))
         text.append("PyGObject: %s" % fver(pygobject_version))
         if player:
             text.append(player.version_info)
         self.set_comments("\n".join(text))
+        self.set_license_type(Gtk.License.GPL_2_0)
         self.set_translator_credits("\n".join(const.TRANSLATORS))
         self.set_website(const.WEBSITE)
         self.set_copyright(const.COPYRIGHT + "\n" +
