@@ -5,13 +5,15 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from gi.repository import Gtk, GLib, Gdk
+from gi.repository import Gtk, GLib, Gdk, GObject
 
 from quodlibet import config
 from quodlibet import qltk
 from quodlibet.qltk import bookmarks
+from quodlibet.qltk import Icons
 from quodlibet import util
 
+from quodlibet.util import connect_obj, connect_destroy
 from quodlibet.qltk.ccb import ConfigCheckMenuItem
 from quodlibet.qltk.sliderbutton import HSlider
 from quodlibet.qltk.tracker import TimeTracker
@@ -31,6 +33,7 @@ class TimeLabel(Gtk.Label):
     def __init__(self, time_=0):
         Gtk.Label.__init__(self)
         self.__widths = {}  # num-chars -> (max-min-width, max-natural-width)
+        self._disabled = False
         self.set_time(time_)
 
     def do_get_preferred_width(self):
@@ -47,7 +50,21 @@ class TimeLabel(Gtk.Label):
     def set_time(self, time_):
         """Set the time in seconds"""
 
-        self.set_text(util.format_time(time_))
+        self._last_time = time_
+        if self._disabled:
+            return
+        self.set_text(util.format_time_display(time_))
+
+    def set_disabled(self, disabled):
+        """Disable the time display temporarily, means there is no meaningful
+        time to show. Re-enabling will show the previous time value
+        """
+
+        self._disabled = disabled
+        if disabled:
+            self.set_text(u"‒\u2236‒‒")
+        else:
+            self.set_time(self._last_time)
 
 
 class SeekBar(HSlider):
@@ -58,6 +75,7 @@ class SeekBar(HSlider):
     def __init__(self, player, library):
         hbox = Gtk.HBox(spacing=3)
         l = TimeLabel()
+        self._time_label = l
         hbox.pack_start(l, True, True, 0)
         arrow = Gtk.Arrow.new(Gtk.ArrowType.RIGHT, Gtk.ShadowType.NONE)
         hbox.pack_start(arrow, False, True, 0)
@@ -66,6 +84,9 @@ class SeekBar(HSlider):
         self._slider_label = TimeLabel()
         self.set_slider_widget(self._slider_label)
 
+        self._on_seekable_changed(player)
+        connect_destroy(player, "notify::seekable", self._on_seekable_changed)
+
         self.scale.connect('button-press-event', self.__seek_lock)
         self.scale.connect('button-release-event', self.__seek_unlock, player)
         self.scale.connect('key-press-event', self.__seek_lock)
@@ -77,11 +98,11 @@ class SeekBar(HSlider):
         c = ConfigCheckMenuItem(
             _("Display remaining time"), "player", "time_remaining")
         c.set_active(config.getboolean("player", "time_remaining"))
-        c.connect_object('toggled', self.scale.emit, 'value-changed')
+        connect_obj(c, 'toggled', self.scale.emit, 'value-changed')
         self.__remaining = c
         m.append(c)
         m.append(SeparatorMenuItem())
-        i = qltk.MenuItem(_("_Edit Bookmarks..."), Gtk.STOCK_EDIT)
+        i = qltk.MenuItem(_(u"_Edit Bookmarks…"), Icons.EDIT)
 
         def edit_bookmarks_cb(menu_item):
             window = bookmarks.EditBookmarks(self, library, player)
@@ -90,15 +111,23 @@ class SeekBar(HSlider):
         i.connect('activate', edit_bookmarks_cb)
         m.append(i)
         m.show_all()
-        self.connect_object(
+
+        connect_obj(self,
             'button-press-event', self.__check_menu, m, player, c)
-        self.connect_object('popup-menu', self.__popup_menu, m, player)
+        connect_obj(self, 'popup-menu', self.__popup_menu, m, player)
 
         timer = TimeTracker(player)
-        timer.connect_object('tick', self.__check_time, player)
+        connect_obj(timer, 'tick', self.__check_time, player)
 
-        player.connect('song-started', self.__song_changed, l, m)
-        player.connect('seek', self.__seeked)
+        connect_destroy(
+            library, "changed", self.__songs_changed, player, m)
+
+        connect_destroy(player, 'song-started', self.__song_started, m)
+        connect_destroy(player, 'seek', self.__seeked)
+
+    def _on_seekable_changed(self, player, *args):
+        self._time_label.set_disabled(not player.seekable)
+        self.set_slider_disabled(not player.seekable)
 
     def __check_menu(self, menu, event, player, remaining_item):
         if event.type != Gdk.EventType.BUTTON_PRESS:
@@ -127,7 +156,7 @@ class SeekBar(HSlider):
                 menu.insert(i, 2)
 
         if event:
-            menu.popup(None, None, None, None, 3, event.time)
+            qltk.popup_menu_at_widget(menu, self, 3, event.time)
             return True
         else:
             time = Gtk.get_current_event_time()
@@ -178,8 +207,16 @@ class SeekBar(HSlider):
         timer.set_time(value)
         self._slider_label.set_time(remaining)
 
-    def __song_changed(self, player, song, label, menu):
+    def __songs_changed(self, library, songs, player, menu):
+        song = player.song
+        if song in songs:
+            self.__update_slider(song, menu)
 
+    def __song_started(self, player, song, menu):
+        self.scale.set_value(0)
+        self.__update_slider(song, menu)
+
+    def __update_slider(self, song, menu):
         if song and song("~#length") > 0:
             self.scale.set_range(0, song("~#length"))
             slider_width = int(song("~#length") / 1.5) + 80
@@ -189,8 +226,6 @@ class SeekBar(HSlider):
             slider_width = 0
             self.__seekable = False
 
-        self.scale.set_value(0)
-
         slider_width = min(max(slider_width, 170), 400)
         self.set_slider_length(slider_width)
 
@@ -202,29 +237,42 @@ class SeekBar(HSlider):
 
 
 class Volume(Gtk.VolumeButton):
-    def __init__(self, device):
+    def __init__(self, player):
         super(Volume, self).__init__(size=Gtk.IconSize.MENU, use_symbolic=True)
 
         self.set_relief(Gtk.ReliefStyle.NORMAL)
         self.set_adjustment(Gtk.Adjustment.new(0, 0, 1, 0.05, 0.1, 0))
 
-        self.connect('value-changed', self.__volume_changed, device)
-        device.connect('notify::volume', self.__volume_notify)
-        self.set_value(config.getfloat("memory", "volume"))
+        self._id = self.connect('value-changed', self.__volume_changed, player)
+        self._id2 = player.connect('notify::volume', self.__volume_notify)
+        self._id3 = player.connect('notify::mute', self.__mute_notify)
+        self._orig_icon_list = self.props.icons
+        player.notify("volume")
+        player.notify("mute")
 
-        replaygain_menu = ReplayGainMenu(device)
+        self.connect("event", self._on_button_event, player)
+
+        replaygain_menu = VolumeMenu(player)
         self.connect('popup-menu', self.__popup, replaygain_menu)
-        self.connect_object('button-press-event', self.__volume_button_press,
-                            replaygain_menu)
+        connect_obj(self, 'button-press-event', self.__volume_button_press,
+                    replaygain_menu, player)
 
     def __popup(self, widget, menu):
         time = Gtk.get_current_event_time()
         button = 3
         return qltk.popup_menu_under_widget(menu, widget, button, time)
 
-    def __volume_button_press(self, menu, event):
+    def __volume_button_press(self, menu, event, player):
+        if event.type != Gdk.EventType.BUTTON_PRESS:
+            return False
+
         if event.button == Gdk.BUTTON_SECONDARY:
-            menu.popup(None, None, None, None, event.button, event.time)
+            qltk.popup_menu_at_widget(menu, self, event.button, event.time)
+            return True
+        elif event.button == Gdk.BUTTON_MIDDLE:
+            # toggle the muted state, if the backend doesn't support it
+            # this action will just be ignored
+            player.mute = not player.mute
             return True
 
     def __iadd__(self, v):
@@ -235,15 +283,40 @@ class Volume(Gtk.VolumeButton):
         self.set_value(self.get_value() - v)
         return self
 
-    def __volume_changed(self, button, volume, device):
-        config.set("memory", "volume", str(volume))
-        device.volume = volume
+    def __volume_changed(self, button, volume, player):
+        player.handler_block(self._id2)
+        player.volume = volume ** 3.0
+        player.handler_unblock(self._id2)
 
-    def __volume_notify(self, device, prop):
-        self.set_value(device.props.volume)
+    def __volume_notify(self, player, prop):
+        self.handler_block(self._id)
+        self.set_value(player.volume ** (1.0 / 3.0))
+        self.handler_unblock(self._id)
+
+    def __mute_notify(self, player, prop):
+        self._update_mute(player)
+
+    def _update_mute(self, player):
+        if player.mute:
+            # remove all icons except the mute one to show a muted state
+            # that is not affected by the volume slider
+            self.props.icons = [self._orig_icon_list[0]]
+        else:
+            self.props.icons = self._orig_icon_list
+
+    def _on_button_event(self, widget, event, player):
+        # pulsesink doesn't emit volume changes when it's paused, but
+        # fetching the value works. To prevent user volume changes based on a
+        # false starting point update the slider on any action on the
+        # volume button.
+        self.handler_block(self._id)
+        self.set_value(player.volume ** (1.0 / 3.0))
+        self.handler_unblock(self._id)
+        # same with mute
+        self._update_mute(player)
 
 
-class ReplayGainMenu(Gtk.Menu):
+class VolumeMenu(Gtk.Menu):
     __modes = (
         ("auto", _("Auto_matic"), None),
         ("track", _("_Track Mode"), ["track"]),
@@ -251,13 +324,29 @@ class ReplayGainMenu(Gtk.Menu):
     )
 
     def __init__(self, player):
-        super(ReplayGainMenu, self).__init__()
+        super(VolumeMenu, self).__init__()
 
+        # ubuntu 12.04..
+        if hasattr(player, "bind_property"):
+            # Translators: player state, no action
+            item = Gtk.CheckMenuItem(label=_("_Mute"), use_underline=True)
+            player.bind_property("mute", item, "active",
+                                 GObject.BindingFlags.BIDIRECTIONAL)
+            self.append(item)
+            item.show()
+
+        item = Gtk.MenuItem(label=_("_Replay Gain Mode"), use_underline=True)
+        self.append(item)
+        item.show()
+
+        rg = Gtk.Menu()
+        rg.show()
+        item.set_submenu(rg)
         item = None
         for mode, title, profile in self.__modes:
             item = RadioMenuItem(group=item, label=title,
                                  use_underline=True)
-            self.append(item)
+            rg.append(item)
             item.connect("toggled", self.__changed, player, profile)
             if player.replaygain_profiles[0] == profile:
                 item.set_active(True)
@@ -266,16 +355,17 @@ class ReplayGainMenu(Gtk.Menu):
     def __changed(self, item, player, profile):
         if item.get_active():
             player.replaygain_profiles[0] = profile
-            player.volume = player.volume
+            player.reset_replaygain()
 
     def popup(self, *args):
         gain = config.getboolean("player", "replaygain")
         for child in self.get_children():
             child.set_sensitive(gain)
-        return super(ReplayGainMenu, self).popup(*args)
+        return super(VolumeMenu, self).popup(*args)
 
 
 class PlayControls(Gtk.VBox):
+
     def __init__(self, player, library):
         super(PlayControls, self).__init__(spacing=3)
 
@@ -322,14 +412,24 @@ class PlayControls(Gtk.VBox):
         self.pack_start(upper, False, True, 0)
         self.pack_start(lower, False, True, 0)
 
-        prev.connect_object('clicked', self.__previous, player)
-        play.connect('toggled', self.__playpause, player)
+        connect_obj(prev, 'clicked', self.__previous, player)
+        self._toggle_id = play.connect('toggled', self.__playpause, player)
         play.add_events(Gdk.EventMask.SCROLL_MASK)
-        play.connect_object('scroll-event', self.__scroll, player)
-        next_.connect_object('clicked', self.__next, player)
-        player.connect('song-started', self.__song_started, next_, play)
-        player.connect_object('paused', play.set_active, False)
-        player.connect_object('unpaused', play.set_active, True)
+        connect_obj(play, 'scroll-event', self.__scroll, player)
+        connect_obj(next_, 'clicked', self.__next, player)
+        connect_destroy(
+            player, 'song-started', self.__song_started, next_, play)
+        connect_destroy(
+            player, 'paused', self.__on_set_paused_unpaused, play, False)
+        connect_destroy(
+            player, 'unpaused', self.__on_set_paused_unpaused, play, True)
+
+    def __on_set_paused_unpaused(self, player, button, state):
+        # block to prevent a signal cycle in case the paused signal and state
+        # get out of sync (shouldn't happen.. but)
+        button.handler_block(self._toggle_id)
+        button.set_active(state)
+        button.handler_unblock(self._toggle_id)
 
     def __scroll(self, player, event):
         if event.direction in [Gdk.ScrollDirection.UP,
