@@ -7,14 +7,18 @@
 
 from gi.repository import Gtk
 from gi.repository import Pango
+import re
 
 from quodlibet import qltk, util
 from quodlibet.util.dprint import print_d
 from quodlibet.qltk.entry import UndoEntry, ValidatingEntry
 from quodlibet.qltk.views import RCMHintedTreeView, HintedTreeView
+from quodlibet.qltk.x import MenuItem, Button
+from quodlibet.qltk import Icons
 from quodlibet.util.json_data import JSONObjectDict
 from quodlibet.util import connect_obj
 from quodlibet.qltk.getstring import GetStringDialog
+from quodlibet.util.tags import _TAGS
 
 
 class JSONBasedEditor(qltk.UniqueWindow):
@@ -72,7 +76,7 @@ class JSONBasedEditor(qltk.UniqueWindow):
 
         # Add context menu
         menu = Gtk.Menu()
-        rem = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_REMOVE, None)
+        rem = MenuItem(_("_Remove"), Icons.LIST_REMOVE)
         keyval, mod = Gtk.accelerator_parse("Delete")
         rem.add_accelerator(
             'activate', self.accels, keyval, mod, Gtk.AccelFlags.VISIBLE)
@@ -85,12 +89,12 @@ class JSONBasedEditor(qltk.UniqueWindow):
 
         # New and Close buttons
         bbox = Gtk.HButtonBox()
-        self.remove_but = Gtk.Button(stock=Gtk.STOCK_REMOVE)
+        self.remove_but = Button(_("_Remove"), Icons.LIST_REMOVE)
         self.remove_but.set_sensitive(False)
-        self.new_but = Gtk.Button(stock=Gtk.STOCK_NEW)
+        self.new_but = Button(_("_New"), Icons.DOCUMENT_NEW)
         self.new_but.connect('clicked', self._new_item)
         bbox.pack_start(self.new_but, True, True, 0)
-        close = Gtk.Button(stock=Gtk.STOCK_CLOSE)
+        close = Button(_("_Close"), Icons.WINDOW_CLOSE)
         connect_obj(close, 'clicked', qltk.Window.destroy, self)
         bbox.pack_start(close, True, True, 0)
         vbox.pack_end(bbox, False, True, 0)
@@ -98,7 +102,6 @@ class JSONBasedEditor(qltk.UniqueWindow):
         self.get_child().pack_start(vbox, True, True, 0)
         # Initialise
         self.selection = view.get_selection()
-        model, _ = self.selection.get_selected()
 
         self.selection.connect('changed', self.__select)
         self.connect('destroy', self.__finish)
@@ -253,13 +256,14 @@ class JSONBasedEditor(qltk.UniqueWindow):
         all.save(filename=self.filename)
 
 
-class MultiStringEditor(qltk.UniqueWindow):
-    """Dialog to edit a list of strings"""
-    _WIDTH = 400
+class TagListEditor(qltk.Window):
+    """Dialog to edit a list of tag names."""
+    _WIDTH = 600
     _HEIGHT = 300
 
     def __init__(self, title, values=None):
-        super(MultiStringEditor, self).__init__()
+        super(TagListEditor, self).__init__()
+        self.use_header_bar()
         self.data = values or []
         self.set_border_width(12)
         self.set_title(title)
@@ -288,7 +292,7 @@ class MultiStringEditor(qltk.UniqueWindow):
 
         # Context menu
         menu = Gtk.Menu()
-        remove_item = Gtk.ImageMenuItem(label=Gtk.STOCK_REMOVE)
+        remove_item = MenuItem(_("_Remove"), Icons.LIST_REMOVE)
         menu.append(remove_item)
         menu.show_all()
         view.connect('popup-menu', self.__popup, menu)
@@ -298,10 +302,10 @@ class MultiStringEditor(qltk.UniqueWindow):
         vbbox = Gtk.VButtonBox()
         vbbox.set_layout(Gtk.ButtonBoxStyle.START)
         vbbox.set_spacing(6)
-        add = Gtk.Button(stock=Gtk.STOCK_ADD)
+        add = Button(_("_Add"), Icons.LIST_ADD)
         add.connect("clicked", self.__add)
         vbbox.pack_start(add, False, True, 0)
-        remove = Gtk.Button(stock=Gtk.STOCK_REMOVE)
+        remove = Button(_("_Remove"), Icons.LIST_REMOVE)
         remove.connect("clicked", self.__remove)
         vbbox.pack_start(remove, False, True, 0)
         hbox.pack_start(vbbox, False, True, 0)
@@ -309,30 +313,53 @@ class MultiStringEditor(qltk.UniqueWindow):
 
         # Close buttons
         bbox = Gtk.HButtonBox()
-        self.remove_but = Gtk.Button(stock=Gtk.STOCK_REMOVE)
+        self.remove_but = Button(_("_Remove"), Icons.LIST_REMOVE)
         self.remove_but.set_sensitive(False)
-        close = Gtk.Button(stock=Gtk.STOCK_CLOSE)
+        close = Button(_("_Close"), Icons.WINDOW_CLOSE)
         connect_obj(close, 'clicked', qltk.Window.destroy, self)
         bbox.set_layout(Gtk.ButtonBoxStyle.END)
-        bbox.pack_start(close, True, True, 0)
-        vbox.pack_start(bbox, False, True, 0)
+        if not self.has_close_button():
+            bbox.pack_start(close, True, True, 0)
+            vbox.pack_start(bbox, False, True, 0)
 
         # Finish up
         self.add(vbox)
         self.get_child().show_all()
 
     def __setup_column(self, view):
-        def cdf(column, cell, model, iter, data):
+        def tag_cdf(column, cell, model, iter, data):
             row = model[iter]
             if row:
                 cell.set_property('text', row[0])
 
+        def desc_cdf(column, cell, model, iter, data):
+            row = model[iter]
+            if row:
+                name = re.sub(':[a-z]+$', '', row[0].strip('~#'))
+                try:
+                    t = _TAGS[name]
+                    valid = (not t.hidden
+                             and t.numeric == row[0].startswith('~#'))
+                    val = t.desc if valid else name
+                except KeyError:
+                    val = name
+                cell.set_property('text', util.title(val.replace('~', ' / ')))
+
+        render = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn(_("Tag expression"), render)
+        column.set_cell_data_func(render, tag_cdf)
+        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+        column.set_expand(True)
+        view.append_column(column)
+
         render = Gtk.CellRendererText()
         render.set_property('ellipsize', Pango.EllipsizeMode.END)
-        column = Gtk.TreeViewColumn(None, render)
-        column.set_cell_data_func(render, cdf)
+        column = Gtk.TreeViewColumn(_("Description"), render)
+        column.set_cell_data_func(render, desc_cdf)
         column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+        column.set_expand(True)
         view.append_column(column)
+        view.set_headers_visible(True)
 
     def __fill_values(self):
         for s in self.data:
@@ -346,8 +373,10 @@ class MultiStringEditor(qltk.UniqueWindow):
         self.view.remove_selection()
 
     def __add(self, *args):
-        dialog = GetStringDialog(self, _("Enter new value"), "",
-                                 okbutton=Gtk.STOCK_ADD)
+        tooltip = _('Tag expression e.g. people:real or ~album~year.')
+        dialog = GetStringDialog(self, _("Enter new tag"), "",
+                                 button_icon=None,
+                                 tooltip=tooltip)
         new = dialog.run()
         if new:
             self.model.append(row=[new])
