@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import os
 import sys
 import time
@@ -6,18 +7,60 @@ import platform
 
 from gi.repository import Gtk
 
+import quodlibet
 from quodlibet import const
 from quodlibet import util
+from quodlibet import qltk
+from quodlibet.qltk.msg import ErrorMessage
+from quodlibet.qltk import Icons
+from quodlibet.qltk import get_top_parent, Align
 from quodlibet.util.path import unexpand, mkdir
+from quodlibet.util import connect_obj
+from quodlibet.util import logging
 
 old_hook = sys.excepthook
 
 
+class MinExceptionDialog(ErrorMessage):
+    """A dialog which shows a title, description and an expandable
+    error report.
+
+    For example decode(traceback.format_exc()) for displaying details
+    about an exception.
+    """
+
+    def __init__(self, parent, title, description, traceback):
+        super(MinExceptionDialog, self).__init__(
+            get_top_parent(parent),
+            title, description)
+
+        assert isinstance(title, unicode)
+        assert isinstance(description, unicode)
+        assert isinstance(traceback, unicode)
+
+        exp = Gtk.Expander(label=_("Error Details"))
+        lab = Gtk.Label(label=traceback)
+        lab.set_alignment(0.0, 0.0)
+        lab.set_selectable(True)
+        win = Gtk.ScrolledWindow()
+        win.add_with_viewport(Align(lab, border=6))
+        win.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        win.set_shadow_type(Gtk.ShadowType.ETCHED_OUT)
+        win.set_size_request(500, 150)
+        win.show_all()
+        exp.add(win)
+        exp.set_resize_toplevel(True)
+
+        area = self.get_message_area()
+        exp.show()
+        area.pack_start(exp, False, True, 0)
+
+
 class ExceptionDialog(Gtk.Window):
     running = False
     instance = None
 
-    DUMPDIR = os.path.join(const.USERDIR, "dumps")
+    DUMPDIR = os.path.join(quodlibet.get_user_dir(), "dumps")
 
     @classmethod
     def from_except(Kind, *args):
@@ -45,9 +88,10 @@ class ExceptionDialog(Gtk.Window):
     @classmethod
     def __dump(self, Kind, value, trace, dump, minidump):
         import mutagen
-        from quodlibet.util import logging
-        dumpobj = file(dump, "w")
-        minidumpobj = file(minidump, "w")
+
+        dumpobj = open(dump, "wb")
+        minidumpobj = open(minidump, "wb")
+
         header = "Quod Libet %s\nMutagen %s\nPython %s %s\nPlatform %s" % (
             const.VERSION, mutagen.version_string, sys.version,
             sys.platform, platform.platform())
@@ -56,14 +100,13 @@ class ExceptionDialog(Gtk.Window):
                          "=== STACK TRACE\n%s\n\n") % (
             header, "\n".join(traceback.format_exception(Kind, value, trace)))
 
-        print >>dumpobj, minidump_data
-        print >>minidumpobj, minidump_data
-
+        dumpobj.write(minidump_data)
+        minidumpobj.write(minidump_data)
         minidumpobj.close()
 
-        for logname in logging.names():
-            print >>dumpobj, "=== LOG: %r\n%s\n\n" % (
-                logname, "\n".join(logging.contents(logname)[-50:]))
+        dumpobj.write("=== LOG:\n")
+        for item in logging.get_content(limit=75):
+            dumpobj.write(item.decode("utf-8") + "\n")
 
         dumpobj.close()
 
@@ -79,14 +122,21 @@ class ExceptionDialog(Gtk.Window):
         self.set_border_width(12)
         self.set_title(_("Error Occurred"))
 
-        desc = _("An exception has occured in Quod Libet. A dump file "
-            "has been saved to <b >%s</b> that will help us debug the crash. "
-            "Please file a new issue at http://code.google.com/p/quodlibet/"
-            "issues/list and attach this file or include its contents. This "
+        desc = _("An exception has occured in Quod Libet. A dump file has "
+            "been saved to <b >%(dump-path)s</b> that will help us debug the "
+            "crash. "
+            "Please file a new issue at %(new-issue-url)s"
+            "and attach this file or include its contents. This "
             "file may contain some identifying information about you or your "
             "system, such as a list of recent files played. If this is "
-            "unacceptable, send <b>%s</b> instead with a description of what "
-            "you were doing.") % (unexpand(dump), unexpand(minidump))
+            "unacceptable, send <b>%(mini-dump-path)s</b> instead with a "
+            "description of what "
+            "you were doing.") % {
+                "dump-path": unexpand(dump),
+                "mini-dump-path": unexpand(minidump),
+                "new-issue-url":
+                    "https://github.com/quodlibet/quodlibet/issues/new",
+            }
 
         suggestion = _("Quod Libet may now be unstable. Closing it and "
             "restarting is recommended. Your library will be saved.")
@@ -106,8 +156,8 @@ class ExceptionDialog(Gtk.Window):
         model = Gtk.ListStore(str, str, int)
         self.__fill_list(view, model, value, traceback)
         view.set_model(model)
-        cancel = Gtk.Button(stock=Gtk.STOCK_CANCEL)
-        close = Gtk.Button(stock=Gtk.STOCK_QUIT)
+        cancel = qltk.Button(_("_Cancel"))
+        close = qltk.Button(_("_Quit"), Icons.APPLICATION_EXIT)
         buttons.pack_start(close, True, True, 0)
         buttons.pack_start(cancel, True, True, 0)
         box.pack_start(label, False, True, 0)
@@ -116,7 +166,7 @@ class ExceptionDialog(Gtk.Window):
         self.add(box)
 
         self.connect('destroy', self.__destroy)
-        cancel.connect_object('clicked', Gtk.Window.destroy, self)
+        connect_obj(cancel, 'clicked', Gtk.Window.destroy, self)
         close.connect('clicked', lambda *x: Gtk.main_quit())
 
         self.get_child().show_all()
