@@ -23,9 +23,10 @@ from quodlibet.qltk.wlw import WritingWindow
 from quodlibet.qltk.window import Dialog
 from quodlibet.qltk.models import ObjectStore
 from quodlibet.qltk.ccb import ConfigCheckButton
-from quodlibet.qltk.x import SeparatorMenuItem
+from quodlibet.qltk.x import SeparatorMenuItem, Button, MenuItem
 from quodlibet.qltk._editutils import EditingPluginHandler, OverwriteWarning
 from quodlibet.qltk._editutils import WriteFailedError
+from quodlibet.qltk import Icons
 from quodlibet.plugins import PluginManager
 from quodlibet.util import connect_obj
 from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
@@ -185,8 +186,8 @@ class SplitValues(Gtk.ImageMenuItem):
     def __init__(self, tag, value):
         super(SplitValues, self).__init__(
             label=_("Split into _Multiple Values"), use_underline=True)
-        self.set_image(Gtk.Image.new_from_stock(
-            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
+        self.set_image(Gtk.Image.new_from_icon_name(
+            Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
         self.set_sensitive(len(split_value(value, spls)) > 1)
@@ -205,8 +206,8 @@ class SplitDisc(Gtk.ImageMenuItem):
     def __init__(self, tag, value):
         super(SplitDisc, self).__init__(
             label=_("Split Disc out of _Album"), use_underline=True)
-        self.set_image(Gtk.Image.new_from_stock(
-            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
+        self.set_image(Gtk.Image.new_from_icon_name(
+            Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
         self.set_sensitive(split_album(value)[1] is not None)
 
     def activated(self, tag, value):
@@ -222,8 +223,8 @@ class SplitTitle(Gtk.ImageMenuItem):
     def __init__(self, tag, value):
         super(SplitTitle, self).__init__(
             label=_("Split _Version out of Title"), use_underline=True)
-        self.set_image(Gtk.Image.new_from_stock(
-            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
+        self.set_image(Gtk.Image.new_from_icon_name(
+            Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
         self.set_sensitive(bool(split_title(value, spls)[1]))
@@ -241,8 +242,8 @@ class SplitPerson(Gtk.ImageMenuItem):
 
     def __init__(self, tag, value):
         super(SplitPerson, self).__init__(label=self.title, use_underline=True)
-        self.set_image(Gtk.Image.new_from_stock(
-            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
+        self.set_image(Gtk.Image.new_from_icon_name(
+            Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
         self.set_sensitive(bool(split_people(value, spls)[1]))
@@ -284,8 +285,9 @@ class AddTagDialog(Dialog):
             use_header_bar=True)
         self.set_border_width(6)
         self.set_resizable(False)
-        self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        add = self.add_button(Gtk.STOCK_ADD, Gtk.ResponseType.OK)
+        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
+        add = self.add_icon_button(_("_Add"), Icons.LIST_ADD,
+                                   Gtk.ResponseType.OK)
         self.vbox.set_spacing(6)
         self.set_default_response(Gtk.ResponseType.OK)
         table = Gtk.Table(n_rows=2, n_columns=2)
@@ -318,8 +320,8 @@ class AddTagDialog(Dialog):
         valuebox.add(hbox)
         hbox.pack_start(self.__val, True, True, 0)
         hbox.set_spacing(6)
-        invalid = Gtk.Image.new_from_stock(
-            Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.SMALL_TOOLBAR)
+        invalid = Gtk.Image.new_from_icon_name(
+            Icons.DIALOG_WARNING, Gtk.IconSize.SMALL_TOOLBAR)
         hbox.pack_start(invalid, True, True, 0)
 
         self.vbox.pack_start(table, True, True, 0)
@@ -396,13 +398,6 @@ class ListEntry(object):
 
 
 class EditTags(Gtk.VBox):
-    _SAVE_BUTTON_KEY = 'ql-save'
-    _REVERT_BUTTON_KEY = 'ql-revert'
-    # Translators: translate only to override the text
-    # for the tag "save" button
-    _SAVE_BUTTON_TEXT = _('ql-save')
-    # Translators: translate only to override the  for the tag "revert" button
-    _REVERT_BUTTON_TEXT = _('ql-revert')
     handler = EditTagsPluginHandler()
 
     @classmethod
@@ -426,11 +421,11 @@ class EditTags(Gtk.VBox):
             if entry.canedit or entry.deleted:
                 rend.set_property('sensitive', entry.edited or entry.deleted)
                 if entry.deleted:
-                    rend.set_property('stock-id', Gtk.STOCK_DELETE)
+                    rend.set_property('icon-name', Icons.EDIT_DELETE)
                 else:
-                    rend.set_property('stock-id', Gtk.STOCK_EDIT)
+                    rend.set_property('icon-name', Icons.EDIT)
             else:
-                rend.set_property('stock-id', Gtk.STOCK_DIALOG_AUTHENTICATION)
+                rend.set_property('icon-name', Icons.DIALOG_PASSWORD)
         column.set_cell_data_func(render, cdf_write)
         view.append_column(column)
 
@@ -490,13 +485,13 @@ class EditTags(Gtk.VBox):
         bbox1 = Gtk.HButtonBox()
         bbox1.set_spacing(6)
         bbox1.set_layout(Gtk.ButtonBoxStyle.START)
-        add = Gtk.Button(stock=Gtk.STOCK_ADD)
+        add = qltk.Button(_("_Add"), Icons.LIST_ADD)
         add.set_focus_on_click(False)
         self._add = add
         add.connect('clicked', self.__add_tag, model, library)
         bbox1.pack_start(add, True, True, 0)
         # Remove button
-        remove = Gtk.Button(stock=Gtk.STOCK_REMOVE)
+        remove = qltk.Button(_("_Remove"), Icons.LIST_REMOVE)
         remove.set_focus_on_click(False)
         remove.connect('clicked', self.__remove_tag, view)
         remove.set_sensitive(False)
@@ -509,17 +504,13 @@ class EditTags(Gtk.VBox):
         bbox2 = Gtk.HButtonBox()
         bbox2.set_spacing(6)
         bbox2.set_layout(Gtk.ButtonBoxStyle.END)
-        revert = (Gtk.Button(stock=Gtk.STOCK_REVERT_TO_SAVED)
-                  if self._REVERT_BUTTON_KEY == self._REVERT_BUTTON_TEXT
-                  else Gtk.Button(label=self._REVERT_BUTTON_TEXT,
-                                  use_underline=True))
+        # Translators: Revert button in the tag editor
+        revert = Button(C_("edittags", "_Revert"), Icons.DOCUMENT_REVERT)
+
         self._revert = revert
         revert.set_sensitive(False)
-        # Save button.
-        save = (Gtk.Button(stock=Gtk.STOCK_SAVE)
-                if self._SAVE_BUTTON_TEXT == self._SAVE_BUTTON_KEY
-                else Gtk.Button(label=self._SAVE_BUTTON_TEXT,
-                                use_underline=True))
+        # Translators: Save button in the tag editor
+        save = Button(C_("edittags", "_Save"), Icons.DOCUMENT_SAVE)
         save.set_sensitive(False)
         self._save = save
         bbox2.pack_start(revert, True, True, 0)
@@ -558,7 +549,7 @@ class EditTags(Gtk.VBox):
         if qltk.is_accel(event, "Delete"):
             self.__remove_tag(view, view)
             return Gdk.EVENT_STOP
-        elif qltk.is_accel(event, "<ctrl>s"):
+        elif qltk.is_accel(event, "<Primary>s"):
             # Issue 697: allow Ctrl-s to save.
             self._save.emit('clicked')
             return Gdk.EVENT_STOP
@@ -638,7 +629,7 @@ class EditTags(Gtk.VBox):
             if menu.get_children():
                 menu.append(SeparatorMenuItem())
 
-        b = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_REMOVE, None)
+        b = MenuItem(_("_Remove"), Icons.LIST_REMOVE)
         b.connect('activate', self.__remove_tag, view)
         qltk.add_fake_accel(b, "Delete")
         menu.append(b)
@@ -660,6 +651,7 @@ class EditTags(Gtk.VBox):
         remove.set_sensitive(bool(rows))
 
     def __add_new_tag(self, model, tag, value):
+        assert isinstance(value, unicode)
         iters = [i for (i, v) in model.iterrows() if v.tag == tag]
         if iters and not self.__songinfo.can_multiple_values(tag):
             title = _("Unable to add tag")
@@ -688,8 +680,11 @@ class EditTags(Gtk.VBox):
                 break
             tag = add.get_tag()
             value = add.get_value()
+            assert isinstance(value, unicode)
             if tag in massagers.tags:
                 value = massagers.tags[tag].validate(value)
+                value = unicode(value)
+            assert isinstance(value, unicode)
             if not self.__songinfo.can_change(tag):
                 title = _("Invalid tag")
                 msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
@@ -771,7 +766,14 @@ class EditTags(Gtk.VBox):
 
             for key, values in deleted.iteritems():
                 for value in values:
-                    if key in song:
+                    if not value.shared:
+                        # In case it isn't shared we don't know the actual
+                        # values to remove. But we know that in that case
+                        # we merge all values into one Comment so just removing
+                        # everything for that key is OK.
+                        song.remove(key, None)
+                        changed = True
+                    elif key in song:
                         song.remove(key, value.text)
                         changed = True
 
@@ -819,15 +821,15 @@ class EditTags(Gtk.VBox):
         new_value = ', '.join(new_value.splitlines())
         path = Gtk.TreePath.new_from_string(path)
         entry = model[path][0]
+        error_dialog = None
 
         if entry.tag in massagers.tags:
             fmt = massagers.tags[entry.tag]
             if not fmt.is_valid(new_value):
-                qltk.WarningMessage(
+                error_dialog = qltk.WarningMessage(
                     self, _("Invalid value"),
                     _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
-                    "value": new_value, "error": fmt.error}).run()
-                return
+                    "value": new_value, "error": fmt.error})
             else:
                 new_value = fmt.validate(new_value)
 
@@ -835,6 +837,10 @@ class EditTags(Gtk.VBox):
         changed = comment.text != new_value
         if (changed and ((comment.shared and comment.complete) or new_value)) \
                 or (new_value and comment.shared and not comment.complete):
+            # only give an error if we would have applied the value
+            if error_dialog is not None:
+                error_dialog.run()
+                return
             entry.value = Comment(new_value)
             entry.edited = True
             entry.deleted = False
