@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig, Michael Urman
 #           2011, 2012 Christoph Reiter
 #
@@ -5,10 +6,14 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from gi.repository import Gtk, GObject, Gdk, Gio
+import math
 
-from quodlibet.qltk.x import is_accel
-from quodlibet.qltk.x import SeparatorMenuItem
+from gi.repository import Gtk, GObject, Gdk, Gio, Pango
+
+from quodlibet.qltk import is_accel, add_fake_accel
+from quodlibet.qltk.x import SeparatorMenuItem, MenuItem
+from quodlibet.qltk import Icons
+from quodlibet.query import Query, QueryType
 
 
 class EditableUndo(object):
@@ -51,10 +56,10 @@ class EditableUndo(object):
             ]
 
     def __key_press(self, entry, event):
-        if is_accel(event, "<ctrl>z"):
+        if is_accel(event, "<Primary>z"):
             self.undo()
             return True
-        elif is_accel(event, "<ctrl><shift>z"):
+        elif is_accel(event, "<Primary><shift>z"):
             self.redo()
             return True
         return False
@@ -70,11 +75,14 @@ class EditableUndo(object):
         del self.__del_pos
 
     def __popup(self, entry, menu):
-        undo = Gtk.ImageMenuItem(Gtk.STOCK_UNDO, use_stock=True)
-        redo = Gtk.ImageMenuItem(Gtk.STOCK_REDO, use_stock=True)
+        undo = MenuItem(_("_Undo"), Icons.EDIT_UNDO)
+        add_fake_accel(undo, "<Primary>z")
+        redo = MenuItem(_("_Redo"), Icons.EDIT_REDO)
+        add_fake_accel(redo, "<Primary><shift>z")
         sep = SeparatorMenuItem()
 
-        map(Gtk.Widget.show, (sep, redo, undo))
+        for widget in [sep, redo, undo]:
+            widget.show()
 
         undo.connect('activate', lambda *x: self.undo())
         redo.connect('activate', lambda *x: self.redo())
@@ -82,7 +90,8 @@ class EditableUndo(object):
         undo.set_sensitive(self.can_undo())
         redo.set_sensitive(self.can_redo())
 
-        map(menu.prepend, (sep, redo, undo))
+        for item in [sep, redo, undo]:
+            menu.prepend(item)
 
     def __all(self):
         text = self.get_chars(0, -1).decode("utf-8")
@@ -141,10 +150,44 @@ class EditableUndo(object):
 
 
 class Entry(Gtk.Entry):
+
+    def __init__(self, *args, **kwargs):
+        super(Entry, self).__init__(*args, **kwargs)
+        self._max_width_chars = -1
+
+        # the default is way too much
+        self.set_width_chars(5)
+
+    def set_max_width_chars(self, value):
+        """Works with GTK+ <3.12"""
+
+        self._max_width_chars = value
+        self.queue_resize()
+
     def do_get_preferred_width(self):
-        # 150 min width since GTK+3.2 is way too much
         minimum, natural = Gtk.Entry.do_get_preferred_width(self)
-        return (30, natural)
+
+        if self._max_width_chars >= 0:
+            # based on gtkentry.c
+            style_context = self.get_style_context()
+            border = style_context.get_border(Gtk.StateFlags.NORMAL)
+            padding = style_context.get_padding(Gtk.StateFlags.NORMAL)
+            pango_context = self.get_pango_context()
+
+            metrics = pango_context.get_metrics(
+                pango_context.get_font_description(),
+                pango_context.get_language())
+
+            char_width = metrics.get_approximate_char_width()
+            digit_width = metrics.get_approximate_digit_width()
+            char_pixels = int(math.ceil(
+                float(max(char_width, digit_width)) / Pango.SCALE))
+
+            space = border.left + border.right + padding.left + padding.right
+            nat_width = self._max_width_chars * char_pixels + space
+            natural = max(nat_width, minimum)
+
+        return (minimum, natural)
 
 
 class UndoEntry(Entry, EditableUndo):
@@ -190,26 +233,22 @@ class ValidatingEntryMixin(object):
     """An entry with visual feedback as to whether it is valid or not.
     The given validator function gets a string and returns True (green),
     False (red), or None (black).
-
-    parse.Query.is_valid_color mimicks the behavior of the search bar.
-
-    If the "Color search terms" option is off, the entry will not
-    change color."""
+    """
 
     INVALID = Gdk.RGBA(0.8, 0, 0)
     VALID = Gdk.RGBA(0.3, 0.6, 0.023)
 
     def set_validate(self, validator=None):
         if validator:
-            self.connect_object('changed', self.__color, validator)
+            self.connect('changed', self.__color, validator)
 
-    def __color(self, validator):
-        value = validator(self.get_text())
+    def __color(self, widget, validator):
+        value = validator(self.get_text().decode("utf-8"))
         if value is True:
             color = self.VALID
         elif value is False:
             color = self.INVALID
-        elif value and isinstance(value, str):
+        elif value and isinstance(value, basestring):
             color = Gdk.RGBA()
             color.parse(value)
         else:
@@ -221,6 +260,20 @@ class ValidatingEntryMixin(object):
             self.override_color(Gtk.StateType.NORMAL, None)
 
 
+def QueryValidator(string):
+    """Returns True/False for a query, None for a text only query"""
+
+    type_ = Query.get_type(string)
+    if type_ == QueryType.VALID:
+        # in case of an empty but valid query we say it's "text"
+        if Query.match_all(string):
+            return None
+        return True
+    elif type_ == QueryType.INVALID:
+        return False
+    return None
+
+
 class ValidatingEntry(ClearEntry, ValidatingEntryMixin):
     def __init__(self, validator=None, *args):
         super(ValidatingEntry, self).__init__(*args)
