@@ -8,40 +8,38 @@
 import os
 
 from gi.repository import Gtk, GObject, Pango
-from quodlibet.qltk.msg import confirm_action
 
 from quodlibet import config
-from quodlibet import const
 from quodlibet import formats
 from quodlibet import qltk
-from quodlibet import util
 
 from quodlibet.plugins import PluginManager
-from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.delete import trash_files, TrashMenuItem
 from quodlibet.qltk.edittags import EditTags
-from quodlibet.qltk.filesel import MainFileSelector, FileSelector
+from quodlibet.qltk.filesel import MainFileSelector
 from quodlibet.qltk.pluginwin import PluginWindow
 from quodlibet.qltk.renamefiles import RenameFiles
 from quodlibet.qltk.tagsfrompath import TagsFromPath
 from quodlibet.qltk.tracknumbers import TrackNumbers
+from quodlibet.qltk.menubutton import MenuButton
 from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.about import AboutExFalso
 from quodlibet.qltk.songsmenu import SongsMenuPluginHandler
-from quodlibet.qltk.x import Alignment, SeparatorMenuItem, ConfigRHPaned
-from quodlibet.qltk.window import PersistentWindowMixin, Window
+from quodlibet.qltk.x import Align, SeparatorMenuItem, ConfigRHPaned, \
+    Button, SymbolicIconImage, MenuItem
+from quodlibet.qltk.window import PersistentWindowMixin, Window, UniqueWindow
+from quodlibet.qltk import Icons
 from quodlibet.util.path import mtime, normalize_path
+from quodlibet.util import connect_obj, connect_destroy
 
 
 class ExFalsoWindow(Window, PersistentWindowMixin):
+
     __gsignals__ = {
-        'changed': (GObject.SignalFlags.RUN_LAST,
-                    None, (object,)),
-        'artwork-changed': (GObject.SignalFlags.RUN_LAST,
-                            None, (object,))
+        'changed': (GObject.SignalFlags.RUN_LAST, None, (object,)),
     }
 
-    pm = SongsMenuPluginHandler(confirm_action)
+    pm = SongsMenuPluginHandler()
 
     @classmethod
     def init_plugins(cls):
@@ -66,28 +64,34 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         bbox = Gtk.HBox(spacing=6)
 
         about = Gtk.Button()
-        about.add(Gtk.Image.new_from_stock(
-            Gtk.STOCK_ABOUT, Gtk.IconSize.BUTTON))
-        about.connect_object('clicked', self.__show_about, self)
+        about.add(Gtk.Image.new_from_icon_name(
+            Icons.HELP_ABOUT, Gtk.IconSize.BUTTON))
+        connect_obj(about, 'clicked', self.__show_about, self)
         bbox.pack_start(about, False, True, 0)
 
-        prefs = Gtk.Button()
-        prefs.add(Gtk.Image.new_from_stock(
-            Gtk.STOCK_PREFERENCES, Gtk.IconSize.BUTTON))
-
-        def prefs_cb(button):
+        def prefs_cb(*args):
             window = PreferencesWindow(self)
             window.show()
-        prefs.connect('clicked', prefs_cb)
-        bbox.pack_start(prefs, False, True, 0)
 
-        plugins = qltk.Button(_("_Plugins"), Gtk.STOCK_EXECUTE)
-
-        def plugin_window_cb(button):
+        def plugin_window_cb(*args):
             window = PluginWindow(self)
             window.show()
-        plugins.connect('clicked', plugin_window_cb)
-        bbox.pack_start(plugins, False, True, 0)
+
+        menu = Gtk.Menu()
+        plugin_item = MenuItem(_("_Plugins"), Icons.SYSTEM_RUN)
+        plugin_item.connect("activate", plugin_window_cb)
+        menu.append(plugin_item)
+
+        pref_item = MenuItem(_("_Preferences"), Icons.PREFERENCES_SYSTEM)
+        pref_item.connect("activate", prefs_cb)
+        menu.append(pref_item)
+        menu.show_all()
+
+        menu_button = MenuButton(
+                SymbolicIconImage(Icons.EMBLEM_SYSTEM, Gtk.IconSize.BUTTON),
+                arrow=True, down=False)
+        menu_button.set_menu(menu)
+        bbox.pack_start(menu_button, False, True, 0)
 
         l = Gtk.Label()
         l.set_alignment(1.0, 0.5)
@@ -97,7 +101,7 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         fs = MainFileSelector()
 
         vb.pack_start(fs, True, True, 0)
-        vb.pack_start(Alignment(bbox, border=6), False, True, 0)
+        vb.pack_start(Align(bbox, border=6), False, True, 0)
         vb.show_all()
 
         hp.pack1(vb, resize=True, shrink=False)
@@ -109,16 +113,15 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
             page = Page(self, self.__library)
             page.show()
             nb.append_page(page)
-        align = Alignment(nb, top=3)
-        align.show()
-        hp.pack2(align, resize=True, shrink=False)
+        hp.pack2(nb, resize=True, shrink=False)
         fs.connect('changed', self.__changed, l)
         if dir:
             fs.go_to(dir)
-        s = self.__library.connect('changed', lambda *x: fs.rescan())
-        self.connect_object('destroy', self.__library.disconnect, s)
+
+        connect_destroy(self.__library, 'changed', self.__library_changed, fs)
+
         self.__save = None
-        self.connect_object('changed', self.set_pending, None)
+        connect_obj(self, 'changed', self.set_pending, None)
         for c in fs.get_children():
             c.get_child().connect('button-press-event',
                 self.__pre_selection_changed, fs, nb)
@@ -131,10 +134,24 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         self.get_child().show()
 
         self.__ag = Gtk.AccelGroup()
-        key, mod = Gtk.accelerator_parse("<control>Q")
+        key, mod = Gtk.accelerator_parse("<Primary>Q")
         self.__ag.connect(key, mod, 0, lambda *x: self.destroy())
         self.add_accel_group(self.__ag)
 
+        # GtkosxApplication assumes the menu bar is mapped, so add
+        # it but don't show it.
+        self._dummy_osx_menu_bar = Gtk.MenuBar()
+        vb.pack_start(self._dummy_osx_menu_bar, False, False, 0)
+
+    def __library_changed(self, library, songs, fs):
+        fs.rescan()
+
+    def set_as_osx_window(self, osx_app):
+        osx_app.set_menu_bar(self._dummy_osx_menu_bar)
+
+    def get_osx_is_persistent(self):
+        return False
+
     def __show_about(self, window):
         about = AboutExFalso(self)
         about.run()
@@ -155,7 +172,6 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
                 return True # cancel or closed
 
     def __popup_menu(self, view, fs):
-
         # get all songs for the selection
         filenames = [normalize_path(f, canonicalise=True)
                      for f in fs.get_selected_paths()]
@@ -163,7 +179,7 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         songs = [s for s in maybe_songs if s]
 
         if songs:
-            menu = self.pm.Menu(self.__library, self, songs)
+            menu = self.pm.Menu(self.__library, songs)
             if menu is None:
                 menu = Gtk.Menu()
             else:
@@ -174,7 +190,11 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         b = TrashMenuItem()
         b.connect('activate', self.__delete, filenames, fs)
         menu.prepend(b)
-        menu.connect_object('selection-done', Gtk.Menu.destroy, menu)
+
+        def selection_done_cb(menu):
+            menu.destroy()
+
+        menu.connect('selection-done', selection_done_cb)
         menu.show_all()
         return view.popup_menu(menu, 0, Gtk.get_current_event_time())
 
@@ -222,7 +242,7 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         self.emit('changed', files)
 
 
-class PreferencesWindow(qltk.UniqueWindow):
+class PreferencesWindow(UniqueWindow):
     def __init__(self, parent):
         if self.is_not_unique():
             return
@@ -242,31 +262,27 @@ class PreferencesWindow(qltk.UniqueWindow):
         l.set_mnemonic_widget(e)
         hb.pack_start(l, False, True, 0)
         hb.pack_start(e, True, True, 0)
-        cb = ConfigCheckButton(
-            _("Show _programmatic tags"), 'editing', 'alltags',
-            tooltip=_("Access all tags, including machine-generated "
-                      "ones e.g. MusicBrainz or Replay Gain tags"))
-        cb.set_active(config.getboolean("editing", 'alltags'))
         vbox.pack_start(hb, False, True, 0)
-        vbox.pack_start(cb, False, True, 0)
         f = qltk.Frame(_("Tag Editing"), child=vbox)
 
-        close = Gtk.Button(stock=Gtk.STOCK_CLOSE)
-        close.connect_object('clicked', lambda x: x.destroy(), self)
+        close = Button(_("_Close"), Icons.WINDOW_CLOSE)
+        connect_obj(close, 'clicked', lambda x: x.destroy(), self)
         button_box = Gtk.HButtonBox()
         button_box.set_layout(Gtk.ButtonBoxStyle.END)
         button_box.pack_start(close, True, True, 0)
 
         main_vbox = Gtk.VBox(spacing=12)
         main_vbox.pack_start(f, True, True, 0)
-        main_vbox.pack_start(button_box, False, True, 0)
+        self.use_header_bar()
+        if not self.has_close_button():
+            main_vbox.pack_start(button_box, False, True, 0)
         self.add(main_vbox)
 
-        self.connect_object('destroy', PreferencesWindow.__destroy, self)
+        connect_obj(self, 'destroy', PreferencesWindow.__destroy, self)
         self.get_child().show_all()
 
     def __changed(self, entry, section, name):
         config.set(section, name, entry.get_text())
 
     def __destroy(self):
-        config.write(const.CONFIG)
+        config.save()
