@@ -9,22 +9,24 @@ import os
 import urlparse
 import errno
 
-from gi.repository import Gtk, GObject, Gdk
+from gi.repository import Gtk, GObject, Gdk, Gio, Pango
 
-from quodlibet import const
 from quodlibet import formats
 from quodlibet import qltk
-from quodlibet import util
 from quodlibet import windows
 
 from quodlibet.qltk.getstring import GetStringDialog
-from quodlibet.qltk.views import AllTreeView, RCMTreeView, MultiDragTreeView
+from quodlibet.qltk.views import AllTreeView, RCMHintedTreeView, \
+    MultiDragTreeView
 from quodlibet.qltk.views import TreeViewColumn
-from quodlibet.qltk.x import ScrolledWindow
+from quodlibet.qltk.x import ScrolledWindow, Paned
 from quodlibet.qltk.models import ObjectStore, ObjectTreeStore
+from quodlibet.qltk import Icons
 
-from quodlibet.util.path import fsdecode, listdir, is_fsnative, glib2fsnative
+from quodlibet.util.path import fsdecode, listdir, is_fsnative, \
+    glib2fsnative, fsnative, xdg_get_user_dirs, get_home_dir
 from quodlibet.util.uri import URI
+from quodlibet.util import connect_obj
 
 
 def search_func(model, column, key, iter_, handledirs):
@@ -36,15 +38,19 @@ def search_func(model, column, key, iter_, handledirs):
     return key not in check.lower() and key not in check
 
 
-def filesel_filter(filename):
+def is_image(filename):
     IMAGES = [".jpg", ".png", ".jpeg"]
+    for ext in IMAGES:
+        if filename.lower().endswith(ext):
+            return True
+    return False
+
+
+def filesel_filter(filename):
     if formats.filter(filename):
         return True
     else:
-        for ext in IMAGES:
-            if filename.lower().endswith(ext):
-                return True
-    return False
+        return is_image(filename)
 
 
 def _get_win_favorites():
@@ -93,7 +99,16 @@ def get_favorites():
     if os.name == "nt":
         return _get_win_favorites()
     else:
-        return [const.HOME, "/"]
+        paths = [get_home_dir()]
+
+        xfg_user_dirs = xdg_get_user_dirs()
+        for key in ["XDG_DESKTOP_DIR", "XDG_DOWNLOAD_DIR", "XDG_MUSIC_DIR"]:
+            if key in xfg_user_dirs:
+                path = xfg_user_dirs[key]
+                if path not in paths:
+                    paths.append(path)
+
+        return paths
 
 
 def _get_win_drives():
@@ -110,7 +125,13 @@ def get_drives():
     if os.name == "nt":
         return _get_win_drives()
     else:
-        return []
+        paths = []
+        for mount in Gio.VolumeMonitor.get().get_mounts():
+            path = mount.get_root().get_path()
+            if path is not None:
+                paths.append(glib2fsnative(path))
+        paths.append("/")
+        return paths
 
 
 def get_gtk_bookmarks():
@@ -122,7 +143,7 @@ def get_gtk_bookmarks():
     if os.name == "nt":
         return []
 
-    path = os.path.join(const.HOME, ".gtk-bookmarks")
+    path = os.path.join(get_home_dir(), ".gtk-bookmarks")
     folders = []
     try:
         with open(path, "rb") as f:
@@ -138,7 +159,7 @@ def get_gtk_bookmarks():
     return folders
 
 
-class DirectoryTree(RCMTreeView, MultiDragTreeView):
+class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
     """A tree view showing multiple folder hierarchies"""
 
     def __init__(self, initial=None, folders=None):
@@ -157,9 +178,12 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
         column = TreeViewColumn(_("Folders"))
         column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         render = Gtk.CellRendererPixbuf()
-        render.set_property('stock_id', Gtk.STOCK_DIRECTORY)
+        render.set_property('icon-name', Icons.FOLDER)
+        render.props.xpad = 3
         column.pack_start(render, False)
         render = Gtk.CellRendererText()
+        if self.supports_hints():
+            render.set_property('ellipsize', Pango.EllipsizeMode.END)
         column.pack_start(render, True)
 
         def cell_data(column, cell, model, iter_, userdata):
@@ -194,20 +218,20 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
             self.go_to(initial)
 
         menu = Gtk.Menu()
-        m = qltk.MenuItem(_("_New Folder..."), Gtk.STOCK_NEW)
+        m = qltk.MenuItem(_(u"_New Folder…"), Icons.DOCUMENT_NEW)
         m.connect('activate', self.__mkdir)
         menu.append(m)
-        m = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_DELETE, None)
+        m = qltk.MenuItem(_("_Delete"), Icons.EDIT_DELETE)
         m.connect('activate', self.__rmdir)
         menu.append(m)
-        m = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_REFRESH, None)
+        m = qltk.MenuItem(_("_Refresh"), Icons.VIEW_REFRESH)
         m.connect('activate', self.__refresh)
         menu.append(m)
-        m = qltk.MenuItem(_("_Select All Subfolders"), Gtk.STOCK_DIRECTORY)
+        m = qltk.MenuItem(_("_Select All Subfolders"), Icons.FOLDER)
         m.connect('activate', self.__expand)
         menu.append(m)
         menu.show_all()
-        self.connect_object('popup-menu', self.__popup_menu, menu)
+        connect_obj(self, 'popup-menu', self.__popup_menu, menu)
 
         # Allow to drag and drop files from outside
         targets = [
@@ -225,7 +249,7 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
         return [model[p][0] for p in paths]
 
     def go_to(self, path_to_go):
-        assert util.is_fsnative(path_to_go)
+        assert is_fsnative(path_to_go)
 
         # FIXME: what about non-normalized paths?
         model = self.get_model()
@@ -233,7 +257,7 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
         # Find the top level row which has the largest common
         # path with the path we want to go to
         roots = dict([(p, i) for (i, p) in model.iterrows(None)])
-        head, tail = path_to_go, util.fsnative(u"")
+        head, tail = path_to_go, fsnative(u"")
         to_find = []
         while head and head not in roots:
             new_head, tail = os.path.split(head)
@@ -429,7 +453,7 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
                 window.set_cursor(None)
 
 
-class FileSelector(Gtk.VPaned):
+class FileSelector(Paned):
     """A file selector widget consisting of a folder tree
     and a file list below.
     """
@@ -446,7 +470,8 @@ class FileSelector(Gtk.VPaned):
         folders -- list of shown folders in the directory tree
         """
 
-        super(FileSelector, self).__init__()
+        super(FileSelector, self).__init__(
+            orientation=Gtk.Orientation.VERTICAL)
         self.__filter = filter
 
         if initial is not None:
@@ -462,10 +487,21 @@ class FileSelector(Gtk.VPaned):
         column = TreeViewColumn(_("Songs"))
         column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         render = Gtk.CellRendererPixbuf()
-        render.set_property('stock_id', Gtk.STOCK_FILE)
         render.props.xpad = 3
+
+        def cell_icon(column, cell, model, iter_, userdata):
+            value = model.get_value(iter_)
+            if is_image(value):
+                cell.set_property('icon-name', Icons.IMAGE_X_GENERIC)
+            else:
+                cell.set_property('icon-name', Icons.AUDIO_X_GENERIC)
+
+        column.set_cell_data_func(render, cell_icon)
+
         column.pack_start(render, False)
         render = Gtk.CellRendererText()
+        if filelist.supports_hints():
+            render.set_property('ellipsize', Pango.EllipsizeMode.END)
         column.pack_start(render, True)
 
         def cell_data(column, cell, model, iter_, userdata):
@@ -596,18 +632,7 @@ class MainFileSelector(FileSelector):
     def __init__(self, initial=None):
         folders = _get_main_folders()
         super(MainFileSelector, self).__init__(
-            initial, self._filesel_filter, folders=folders)
-
-    @staticmethod
-    def _filesel_filter(filename):
-        IMAGES = [".jpg", ".png", ".jpeg"]
-        if formats.filter(filename):
-            return True
-        else:
-            for ext in IMAGES:
-                if filename.lower().endswith(ext):
-                    return True
-        return False
+            initial, filesel_filter, folders=folders)
 
 
 class MainDirectoryTree(DirectoryTree):
